from . import сегментация, повышение_размерности, обнаружение, обработка_текста, модель,квартиры, home, traide, check_for_errors
import subprocess
from subprocess import STDOUT, check_call
import os, time, random, re,pymorphy2, shutil
import matplotlib.pyplot as plt
import numpy as np
from tensorflow.keras.preprocessing import image
from IPython import display
from tensorflow.keras.utils import to_categorical, plot_model # Полкючаем методы .to_categorical() и .plot_model()
from tensorflow.keras import datasets # Подключаем набор датасетов
import importlib.util, sys, gdown
from tqdm.notebook import tqdm_notebook as tqdm_
from PIL import Image
import pandas as pd
import requests
import ast
import json

import zipfile
from sklearn.model_selection import train_test_split

#sda#sda
###
###                 ЗАГРУЗКА ДАННЫХ
###
def загрузить_базу(база = ''):
  справка = True
  база_1 = база
  база = база.lower()
  print('Загрузка данных')
  print('Это может занять несколько минут...')  
  if база == 'mnist':
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    if справка:
      print('Вы скачали базу рукописных цифр MNIST. \nБаза состоит из двух наборов данных: обучающего (60тыс изображений) и тестовго (10тыс изображений).')
      print('Размер каждого изображения: 28х28 пикселей')
  
  elif база == 'авто-2':
    загрузить_базу_АВТОМОБИЛИ_2()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу с изображениями марок автомобилей. \nБаза состоит из двух марок: Феррари и Мерседес')
      print('Количество изображений в базе: 2249') 

  elif база == 'авто-3':
    загрузить_базу_АВТОМОБИЛИ_3()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу с изображениями марок автомобилей. \nБаза состоит из трех марок: Феррари, Мерседес и Рено')
      print('Количество изображений в базе: 3429')

  elif база == 'молочная_продукция':
    загрузить_базу_МОЛОКО()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу с изображениями бутылок молока')

  elif база == 'автобусы':
    загрузить_базу_АВТОБУСЫ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу с изображениями входящих и выходящих пассажиров в автобус')
  
  elif база == 'tesla':
    загрузить_базу_TESLA()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу отзывов владельцев автомобилями Tesla')

  elif база == 'обнаружение_возгораний':
    загрузить_базу_ОГОНЬ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу для классификации огня.')

  elif база == 'майонез':
    загрузить_базу_МАЙОНЕЗ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу для классификации майонеза.')

  elif база == 'трекер':
    загрузить_базу_ТРЕКЕР()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу для трекинга пассажиров в автобусе.')

  elif база == 'вакансии':
    загрузить_базу_ВАКАНСИИ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу вакансий.')

  elif база == 'самолеты':
    загрузить_базу_САМОЛЕТЫ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу самолетов. \nБаза состоит из оригинальных изображений и соовтетствующих им размеченных сегментированных изображений.')
      print('Количество изображений в базе: 981')    

  elif база == 'самолеты_макс':
    загрузить_базу_САМОЛЕТЫ_макс()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу самолетов. \nБаза состоит из оригинальных изображений и соовтетствующих им размеченных сегментированных изображений.')
      print('Количество изображений в базе: 981') 
  
  elif база == 'люди':
    загрузить_базу_Люди()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу на которой изображены люди. \nБаза состоит из оригинальных изображений и соответствующих им размеченных сегментированных изображений.')


  elif база == 'кожные_заболевания':
    загрузить_базу_БОЛЕЗНИ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу кожных заболеваний. \nБаза состоит из оригинальных изображений и соовтетствующих им размеченных сегментированных изображений.')
      print('Количество категорий заболеваний: 10 (Акне, Витилиго, Герпес, Дерматит, Лишай, Невус, Псориаз, Сыпь, Хлоазма, Экзема)')    
      print('Количество изображений в базе: 981')    
  
  elif база == 'повышение_размерности':
    загрузить_базу_HR()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Вы скачали базу изображений для задачи повышения размерности')
      print('База содержит изображения высокого качества и соответствующие им изображения низкого качества')

  elif база == 'обнаружение_людей':
    загрузить_базу_ЛЮДЕЙ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена размеченная база изображений для обнаружения людей')      
    print()
    print('ВНИМАНИЕ!!! Были установлены дополнительные библиотеки. Необходимо перезапустить среду для продолжения работы')
    print('Выберите пункт меню Runtime/Restart runtime и нажмите «Yes»')
    print('После этого сделайте повторный запуск ячейки: import terra_ai')
  
  elif база == 'симптомы_заболеваний':
    #Загрузка базы
    загрузить_базу_ЗАБОЛЕВАНИЯ()
    #display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база симптомов заболеваний') 
    print()
    
  elif база == 'писатели':
    загрузить_базу_ПИСАТЕЛИ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база писателей')      
    print()

  elif база == 'диалоги':
    загрузить_базу_ДИАЛОГИ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база диалогов')
      print('Количество пар вопрос-ответ: 50 тысяч')
    print()
  elif база == 'договоры':
    загрузить_базу_ДОГОВОРА()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база договоров')
      print('База размечена по 6 категориям: Условия - Запреты - Стоимость - Деньги - Сроки - Неустойка')
# s6 Всё про адреса и геолокации
    print()

  elif база == 'квартиры':
    загрузить_базу_КВАРТИРЫ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база квартир')
    print()

  elif база == 'губы':
    загрузить_базу_ГУБЫ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база сегментации женских губ')
    print()

  elif база == 'трафик':
    загрузить_базу_ТРАФИК()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база трафика компании')
    print()

  elif база == 'умный_дом':
    загрузить_базу_УМНЫЙ_ДОМ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружена база голосовых команд для умного дома')
    print()

  elif база == 'трейдинг':
    загрузить_базу_ТРЕЙДИНГ()
    display.clear_output(wait=True)
    print('Загрузка данных завершена \n')
    print('url:', url)
    if справка:
      print('Загружены базы акций трех вариантов: полиметаллы, газпром и яндекс')
    print()
  else:
    display.clear_output(wait=True)
    assert False, f"Указанная база '{база_1}' не найдена. Возможно вы имели ввиду '{check_for_errors.check_word(база, 'dataset')}'.\n"

def загрузить_базу_конструктора():
  url = 'https://storage.googleapis.com/aiu_bucket/UI.zip' # Указываем URL-файла
  output = 'UI.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

def загрузить_базу_ТРАФИК():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/traff.csv'
  output = 'traff.csv' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True)

def загрузить_базу_ТРЕЙДИНГ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/shares.zip'
  output = 'shares.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) 
  if not os.path.exists('/content/трейдинг/'): 
      распаковать_архив(откуда = "shares.zip",куда = "/content/")
      os.rename('/content/shares','/content/трейдинг')
  

def загрузить_базу_АВТОМОБИЛИ_2():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/car_2.zip'
  output = 'car.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "car.zip",
      куда = "/content/авто-2"
  )

def загрузить_базу_АВТОМОБИЛИ_3():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/car.zip'
  output = 'car.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "car.zip",
      куда = "/content/авто-3"
  )

def загрузить_базу_МОЛОКО():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/milk.zip'
  output = 'milk.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "milk.zip",
      куда = "/content/молочная_продукция"

  )
def загрузить_базу_АВТОБУСЫ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/bus.zip'
  output = 'bus.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "bus.zip",
      куда = "/content/автобусы"
  )

def загрузить_базу_ОГОНЬ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/fire.zip'
  output = 'fire.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  if not os.path.exists('/content/болезни'): 
      распаковать_архив(откуда = "fire.zip",куда = "/content/")
      os.rename('/content/Обнаружение_возгораний','/content/обнаружение_возгораний')

def загрузить_базу_МАЙОНЕЗ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/mayonnaise.zip'
  output = 'mayonnaise.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "mayonnaise.zip",
      куда = "/content/майонез"
  )  

def загрузить_базу_ТРЕКЕР():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/Heads.zip'
  output = 'Heads.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "Heads.zip",
      куда = "/content/трекер"
  )   
def загрузить_базу_ВАКАНСИИ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/HR.zip'
  output = 'HR.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "HR.zip",
      куда = "/content/вакансии")

def загрузить_базу_УМНЫЙ_ДОМ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/cHome.zip'
  output = 'cHome.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "cHome.zip",
      куда = "/content/умный_дом"
  )

def загрузить_базу_КВАРТИРЫ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/moscow.csv'
  output = 'moscow.csv' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

def загрузить_базу_ДИАЛОГИ():
  global url
  #обнаружение.выполнить_команду('mkdir content')
  url = 'https://storage.googleapis.com/terra_ai/DataSets/dialog.txt'
  output = 'dialog.txt' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  

def загрузить_базу_ЗАБОЛЕВАНИЯ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/symptoms.zip'
  output = 'symptoms.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  # Скачиваем и распаковываем архив
  if not os.path.exists('/content/болезни'): 
    распаковать_архив(откуда = "symptoms.zip",куда = "/content/")
    os.rename('/content/Болезни','/content/симптомы_заболеваний')

def загрузить_базу_TESLA(): 
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/tesla.zip'
  output = 'tesla.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "tesla.zip",
      куда = "/content/tesla"
  )

def загрузить_базу_ПИСАТЕЛИ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/writers.zip'
  output = 'writers.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "writers.zip",
      куда = "content/"
  )
  
def загрузить_базу_ЛЮДЕЙ():
  global url
  url = 'https://github.com/ultralytics/yolov5/archive/master.zip'
  output = 'tmp.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  распаковать_архив(
      откуда = "tmp.zip",
      куда = "/content"
  )
  # Скачиваем и распаковываем архив
  url = 'https://github.com/ultralytics/yolov5/releases/download/v1.0/coco2017val.zip'
  output = 'tmp.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  распаковать_архив(
      откуда = "tmp.zip",
      куда = "/content"
  )

  url = 'https://github.com/ultralytics/yolov5/releases/download/v1.0/coco128.zip'
  output = 'tmp.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  распаковать_архив(
      откуда = "tmp.zip",
      куда = "/content"
  )
  обнаружение.выполнить_команду('echo y|pip uninstall albumentations > /dev/null')
  обнаружение.выполнить_команду('pip install -q --no-input -U git+https://github.com/albumentations-team/albumentations > /dev/null')
  обнаружение.выполнить_команду('pip install -q -U PyYAML')
  
  
def загрузить_базу_ДОГОВОРА():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/docs.zip'
  output = 'договоры.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL

  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "договоры.zip",
      куда = "/content/договоры"
      )
 
  url = 'https://storage.googleapis.com/terra_ai/DataSets/test_doc.txt'
  output = '/content/договоры/test_doc.txt' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL


def загрузить_договоры():

  def readText(fileName):
    f = open(fileName, 'r', encoding='utf-8') #Открываем наш файл для чтения и считываем из него данные 
    text = f.read() #Записываем прочитанный текст в переменную 
    delSymbols = ['\n', "\t", "\ufeff", ".", "_", "-", ",", "!", "?", "–", "(", ")", "«", "»", "№", ";"]
    for dS in delSymbols: # Каждый символ в списке символов для удаления
      text = text.replace(dS, " ") # Удаляем, заменяя на пробел
    text = re.sub("[.]", " ", text) 
    text = re.sub(":", " ", text)
    text = re.sub("<", " <", text)
    text = re.sub(">", "> ", text)
    text = ' '.join(text.split()) 
    return text # Возвращаем тексты
  def text2Words(text):
    morph = pymorphy2.MorphAnalyzer() # Создаем экземпляр класса MorphAnalyzer
    words = text.split(' ') # Разделяем текст на пробелы
    words = [morph.parse(word)[0].normal_form for word in words] #Переводим каждое слово в нормалную форму  
    return words # Возвращаем слова   
  directory = 'договоры/Договора432/' # Путь к папке с договорами
  agreements = [] # Список, в который запишем все наши договоры
  for filename in os.listdir(directory): # Проходим по всем файлам в директории договоров
    try:    
        txt = readText(directory + filename) # Читаем текст договора
        if txt != '': # Если текст не пустой
          agreements.append(readText(directory + filename)) # Преобразуем файл в одну строку и добавляем в agreements
    except:
        continue
  words = [] # Здесь будут храниться все договора в виде списка слов
  curTime = time.time() # Засечем текущее время
  for i in tqdm_(range(len(agreements)), desc='Обработка догововров', ncols=1000): # Проходимся по всем договорам
    words.append(text2Words(agreements[i])) # Преобразуем очередной договор в список слов и добавляем в words
  wordsToTest = words[-10:] # Возьмем 10 текстов для финальной проверки обученной нейронной сети 
  words = words[:-10] # Для обученающей и проверочной выборок возьмем все тексты, кроме последних 10
  display.clear_output(wait=True)
  return (agreements, words, wordsToTest)  #, agreements

def загрузить_базу_САМОЛЕТЫ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/airplane.zip'
  output = 'самолеты.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  url = 'https://storage.googleapis.com/terra_ai/DataSets/segment.zip' # Указываем URL-файла
  output = 'сегменты.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  
  # Скачиваем и распаковываем архив
  if not os.path.exists('/content/самолеты'): 
      распаковать_архив(откуда = "самолеты.zip",куда = "/content/самолеты/")
      os.rename('/content/самолеты/Самолеты','/content/самолеты/оригинал')
      
      распаковать_архив(откуда = "сегменты.zip",куда = "/content/самолеты/")
      os.rename('/content/самолеты/Сегменты','/content/самолеты/сегментация')

  display.clear_output(wait=True)  
  # Обрабатываем скаченные изображения

def загрузить_базу_САМОЛЕТЫ_макс():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/airplane_max.zip' # Указываем URL-файла
  output = 'airplane_max.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL


      
      
  url = 'https://storage.googleapis.com/terra_ai/DataSets/segment_max.zip' # Указываем URL-файла
  output = 'segment_max.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  if not os.path.exists('/content/самолеты_макс/'): 
  
      распаковать_архив(откуда = "airplane_max.zip",куда = "/content/самолеты_макс/")
      os.rename('/content/самолеты_макс/airplane_max','/content/самолеты_макс/оригинал')
      
      распаковать_архив(откуда = "segment_max.zip",куда = "/content/самолеты_макс/")
      os.rename('/content/самолеты_макс/segment_max','/content/самолеты_макс/сегментация')
  display.clear_output(wait=True)  
  # Обрабатываем скаченные изображения

def загрузить_базу_Люди():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/original_people.zip' # Указываем URL-файла
  output = 'original_people.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "original_people.zip",
      куда = "/content"
  )
  url = 'https://storage.googleapis.com/terra_ai/DataSets/segment_people.zip' # Указываем URL-файла
  output = 'segment_people.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "segment_people.zip",
      куда = "/content"
  )

  if not os.path.exists('/content/люди/'): 
  
      распаковать_архив(откуда = "original_people.zip",куда = "/content/люди/")
      os.rename('/content/люди/original','/content/люди/оригинал')
      
      распаковать_архив(откуда = "segment_people.zip",куда = "/content/люди/")
      os.rename('/content/люди/segment','/content/люди/сегментация')
  display.clear_output(wait=True)  
  # Обрабатываем скаченные изображения
  
def загрузить_базу_ГУБЫ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/lips.zip' # Указываем URL-файла
  output = 'lips.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "lips.zip",
      куда = "/content"
  )
  if not os.path.exists('/content/губы'): 
      распаковать_архив(откуда = "lips.zip",куда = "/content/")
      os.rename('/content/Губы','/content/губы')
  display.clear_output(wait=True)  

def загрузить_базу_БОЛЕЗНИ():
  global url
  url = 'https://storage.googleapis.com/terra_ai/DataSets/origin.zip'
  output = 'diseases.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "diseases.zip",
      куда = "/content/diseases"
  )
  url = 'https://storage.googleapis.com/terra_ai/DataSets/segmentation.zip'
  output = 'segm.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "segm.zip",
      куда = "/content/diseases"
  )

def загрузить_базу_HR():
  global url
  # Скачиваем и распаковываем архив в колаб по ссылке
  url = 'http://data.vision.ee.ethz.ch/cvl/DIV2K/DIV2K_valid_LR_bicubic_X4.zip'          
  output = 'DIV2K_valid_LR_bicubic_X4.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "DIV2K_valid_LR_bicubic_X4.zip",
      куда = "/content/повышение_размерности"
  )
  url = 'http://data.vision.ee.ethz.ch/cvl/DIV2K/DIV2K_valid_HR.zip'
  output = 'DIV2K_valid_HR.zip' # Указываем имя файла, в который сохраняем файл
  gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
  распаковать_архив(
      откуда = "DIV2K_valid_HR.zip",
      куда = "/content/повышение_размерности"
  )
  os.system("cp -r /content/повышение_размерности/DIV2K_valid_LR_bicubic/X4 /content/повышение_размерности")
  shutil.rmtree('//content/повышение_размерности/DIV2K_valid_LR_bicubic')
  os.rename('/content/повышение_размерности/X4', '/content/повышение_размерности/DIV2K_valid_LR_bicubic')

def формирование_повышение_размерности():
  pathHR = '/content/повышение_размерности/DIV2K_valid_HR/'
  pathLR = '/content/повышение_размерности/DIV2K_valid_LR_bicubic/'
  imagesList = [os.listdir(pathHR), os.listdir(pathLR)] # Список из корневых каталогов изображений, их два - hr и lr
  train_hr = []
  train_lr = []

  # В цикле проходимся по каталогу с hr изображениями
  for image in sorted(imagesList[0]):
      img = Image.open(pathHR + image)
      train_hr.append(img) # Добавляем все изображения в список

  # В цикле проходимся по каталогу с lr изображениями
  for image in sorted(imagesList[1]):
      img = Image.open(pathLR + image)
      train_lr.append(img) # Добавляем все изображения в список
  return train_hr, train_lr

###
###                    ДЕМОНСТРАЦИЯ ПРИМЕРОВ
###

def показать_примеры(**kwargs):
  kwargs['путь'] =  kwargs['путь'].replace('/','')
  if 'вопросы' in kwargs:  
    count = 1
    if 'количество' in kwargs:
        count = kwargs['количество']
    questions = kwargs['вопросы']
    answers = kwargs['ответы']
    for i in range(count):
        print()
        idx = np.random.randint(len(questions)-1)
        print('Вопрос:', questions[idx])
        print('Ответ:', answers[idx])

  if 'путь' in kwargs:
    kwargs['путь'] = kwargs['путь'].lower() 
    new_data = ['молочная_продукция', 'автобусы','обнаружение_возгораний']
    
    if kwargs['путь'] == 'диалоги':
      assert os.path.exists('/content/' + 'dialog.txt'), f'Загрузите базу для \'{kwargs["путь"]}\' для корректной работы.'  
      f = open('dialog.txt', 'r', encoding='utf-8')
      text= f.read()
      text = text.replace('"','')
      text = text.split('\n')
      question = text[::3]
      answers = text[1::3]
      for i in range(10,0,-1):
        print(question[-i], answers[-i])
    
    elif kwargs['путь'] == 'обнаружение_возгораний':
      assert os.path.exists('/content/' + "обнаружение_возгораний"), f'Загрузите базу  для \'обнаружение_возгораний\' для корректной работы.' 
      path = '/content'
      size=(512,512)
      fire_idx = [20,30,70,80,90,
                  100,130,200,300,500,
                  610,18,19,21,26]
      not_fire_idx = [0,30,80,110,140,
                      320,390,400,450,500,
                      550,840,850,960,1010]

      fig = plt.figure(figsize=(16,8))

      fire_img = sorted(os.listdir(os.path.join(path, 'обнаружение_возгораний/картинки_огня')))
      for i, idx in enumerate(np.random.choice(fire_idx, 4, replace=False)):
          img_path = os.path.join(path, 'обнаружение_возгораний/картинки_огня', fire_img[idx])
          img = image.load_img(img_path, target_size=size)
          img = np.array(img)
          
          ax = fig.add_subplot(2, 4, i+1, xticks=[], yticks=[])
          ax.set_title('Огонь')
          plt.imshow(img)

      not_fire_img = sorted(os.listdir(os.path.join(path, 'обнаружение_возгораний/случайные_картинки')))
      for i, idx in enumerate(np.random.choice(not_fire_idx, 4, replace=False)):
          img_path = os.path.join(path, 'обнаружение_возгораний/случайные_картинки', not_fire_img[idx])
          img = image.load_img(img_path, target_size=size)
          img = np.array(img)
          
          ax = fig.add_subplot(2, 4, i+5, xticks=[], yticks=[])
          ax.set_title('Нет огня')
          plt.imshow(img)
    
    elif kwargs['путь'] == 'трейдинг':
      kwargs.pop('путь')
      traide.show_data(**kwargs)
    elif kwargs['путь'] == 'умный_дом':
       if 'файл' in kwargs:
         home.параметризация_аудио(kwargs['файл'])
       else:
         home.показать_примеры_голосовых_команд()
    
    elif kwargs['путь'] == 'молочная_продукция':
      assert os.path.exists('/content/' + kwargs['путь']), f'Загрузите базу для \'{kwargs["путь"]}\' для корректной работы.'   
      size=(512,256)
      path = '/content/' + kwargs['путь']
      fig = plt.figure(figsize=(10,10))

      classes = sorted(os.listdir(path))
      for i in range(len(classes)):
          images = sorted(os.listdir(os.path.join(path, classes[i])))
          img_path = os.path.join(path, classes[i], images[0])
          img = image.load_img(img_path, target_size=size)
          img = np.array(img)
          
          ax = fig.add_subplot(1, 3, i+1, xticks=[], yticks=[])
          plt.imshow(img)

    elif kwargs['путь'] == 'автобусы':
      assert os.path.exists('/content/' + kwargs['путь']), f'Загрузите базу для \'{kwargs["путь"]}\' для корректной работы.'  
      path = '/content/' + kwargs['путь']
      size=(256,150)
      entering_idx = [1,170,338,1200,1536,
                      1762,1830,1942,2312,2414,
                      2455,2712,3513,3657,3744,
                      3761,3792,3797,3826,3877,
                      3904,3945,3972,3993,4140]

      getting_off_idx = [184,445,653,1220,1241,
                        1283,1367,1388,1409,1453,
                        1487,1648,1906,2505,2412,
                        2292,1865]

      fig = plt.figure(figsize=(16,10))

      entering_img = sorted(os.listdir(os.path.join(path, 'entering')))
      for i, idx in enumerate(np.random.choice(entering_idx, 5, replace=False)):
          img_path = os.path.join(path, 'entering', entering_img[idx])
          img = image.load_img(img_path, target_size=size)
          img = np.array(img)
          
          ax = fig.add_subplot(2, 5, i+1, xticks=[], yticks=[])
          ax.set_title('Входящий')
          plt.imshow(img)

      getting_off_img = sorted(os.listdir(os.path.join(path, 'getting_off')))
      for i, idx in enumerate(np.random.choice(getting_off_idx, 5, replace=False)):
          img_path = os.path.join(path, 'getting_off', getting_off_img[idx])
          img = image.load_img(img_path, target_size=size)
          img = np.array(img)
          
          ax = fig.add_subplot(2, 5, i+6, xticks=[], yticks=[])
          ax.set_title('Выходящий')
          plt.imshow(img)
    elif kwargs['путь'] == 'квартиры':
        if 'количество' in kwargs:
          квартиры.samples(kwargs['количество'])
        else:
          квартиры.samples(5)
    
    elif kwargs['путь'] == 'mnist':

        (_, _), (x_test, y_test) = datasets.mnist.load_data()
        real_img = [x_test[y_test==i][0] for i in range(10)]
        real_img_concat = np.concatenate([i for i in real_img], axis=1)
        plt.figure(figsize=(56, 2*len(real_img)), dpi=25)
        plt.imshow(real_img_concat, cmap='Greys_r')
        plt.grid(False)
        plt.axis('off')
        plt.show()

    elif kwargs['путь'] == 'симптомы_заболеваний':        
        path = f'/content/{kwargs["путь"]}/'
        assert os.path.exists(path), f'Загрузите базу \'{kwargs["путь"]}\'.'
        text = []
        classes = []
        n = 0
        codecs_list = ['UTF-8', 'Windows-1251']

        for filename in sorted(os.listdir(path)): # Проходим по всем файлам в директории договоров
            n +=1
            for codec_s in codecs_list:
                try:
                    text.append(обработка_текста.readText(path+filename, codec_s)) # Преобразуем файл в одну строку и добавляем в agreements
                    classes.append(filename.replace(".txt", ""))
                    break
                except UnicodeDecodeError:
                    print('Не прочитался файл: ', path+currdir+'/'+filename, codec_s)
                else:
                    next 
        nClasses = len(classes) #определяем количество классов
        print('В данной базе содержатся симптомы следующих заболеваний:')
        print(classes)        
        n = np.random.randint(10)
        print()
        print('Пример симптомов случайного заболевания:')
        print('Заболевание: ', classes[n])
        print('Симптомы:')
        print('     *', text[n][:100]) # Пример первых 100 символов первого документа      
        return classes, nClasses

    elif kwargs['путь'] == 'tesla':        
        path = f'/content/{kwargs["путь"]}/'
        assert os.path.exists(path), f'Загрузите базу \'{kwargs["путь"]}\'.'
        text = []
        classes = []
        n = 0
        codecs_list = ['UTF-8', 'Windows-1251']

        for filename in sorted(os.listdir(path)): # Проходим по всем файлам в директории договоров
            n +=1
            for codec_s in codecs_list:
                try:
                    text.append(обработка_текста.readText(path+filename, codec_s)) # Преобразуем файл в одну строку и добавляем в agreements
                    classes.append(filename.replace(".txt", ""))
                    break
                except UnicodeDecodeError:
                    print('Не прочитался файл: ', path+currdir+'/'+filename, codec_s)
                else:
                    next 
        nClasses = len(classes) #определяем количество классов
        print('В данной базе находятся положительные и негативные отзывы об автомобилях Tesla:')
        print(classes)        
        n = np.random.randint(2)
        print()
        print('Пример отзыва:')
        print('Тип отзыва: ', classes[n])
        print('Отзывы:')
        print('     *', text[n][:100]) # Пример первых 100 символов первого документа      
        return classes, nClasses

    elif kwargs['путь'] == 'писатели':        
        path = 'content/writers/'
        assert os.path.exists(path), f'Загрузите базу \'{kwargs["путь"]}\'.'
        text = []
        classes = []
        n = 0
        codecs_list = ['UTF-8', 'Windows-1251']

        for filename in os.listdir(path): # Проходим по всем файлам в директории договоров
            n +=1
            for codec_s in codecs_list:
                try:
                    text.append(обработка_текста.readText(path+filename, codec_s)) # Преобразуем файл в одну строку и добавляем в agreements
                    classes.append(filename.replace(".txt", ""))
                    break
                except UnicodeDecodeError:
                    print('Не прочитался файл: ', path+currdir+'/'+filename, codec_s)
                else:
                    next 
        nClasses = len(classes) #определяем количество классов
        print('В данной базе содержатся произведения следующих писателей:')
        print(classes)        
        n = np.random.randint(6)
        print()
        print('Пример текста случайного произведения и автора:')
        print('Автор: ', classes[n])
        print('Произведение и пример: ', text[n][:78], '\n', text[n][78:178], '\n', text[n][178:278], '\n',
                                         text[n][278:378], '\n', text[n][378:478], '\n', text[n][578:678], '\n', 
                                         text[n][778:878], '\n', text[n][978:1078]) # Пример первых 1000 символов первого документа      
        return classes, nClasses

    elif kwargs['путь'] == 'трекер':               
        path = '/content/трекер'
        kwargs["тип_изображений"] = kwargs["тип_изображений"].lower()
        assert os.path.exists(path), f'Загрузите базу \'{kwargs["путь"]}\'.'
        assert 'тип_изображений' in kwargs, f'Уточните тип изображений параметром \'тип_изображений\'.'
        list_of_humans = ['один человек', 'разные люди']
        
        assert kwargs['тип_изображений'] in list_of_humans, f'Тип изображения \'{kwargs["тип_изображений"]}\' не распознано.\
        Возможно вы имели ввиду \'{check_for_errors.check_word(kwargs["тип_изображений"],extra=list_of_humans)}\'.'
        
        size=(64,64)
        heads_idx = [9,39,100,130,
                    170,330,400,
                    450,500,630,680,700,
                    730,750,810,870,
                    960,1010,1040,1100,1140,
                    1210,1290,1370,1650]

        fig = plt.figure(figsize=(18,7))

        heads_img = sorted(os.listdir(os.path.join(path, 'images')))
        np.random.choice(heads_idx, 5, replace=False)
        for i, idx in enumerate(np.random.choice(heads_idx, 5, replace=False)):
            img_path = os.path.join(path, 'images', heads_img[idx])
            img = image.load_img(img_path, target_size=size)
            img = np.array(img)
            
            ax = fig.add_subplot(2, 5, i+1, xticks=[], yticks=[])
            plt.imshow(img)

            if kwargs['тип_изображений'] == 'один человек':
                next_idx = idx + 3
            elif kwargs['тип_изображений'] == 'разные люди':
                next_idx = np.random.choice(heads_idx, 1)[0]
                while idx == next_idx:
                    next_idx = np.random.choice(heads_idx, 1)[0]
                
            img_path = os.path.join(path, 'images', heads_img[next_idx])
            img = image.load_img(img_path, target_size=size)
            img = np.array(img)

            ax = fig.add_subplot(2, 5, i+6, xticks=[], yticks=[])
            plt.imshow(img)

    elif kwargs['путь'] == 'обнаружение':
      обнаружение.показать_примеры()

    elif kwargs['путь'] == 'трафик':
      dataframe = pd.read_csv('traff.csv', header=None)
      dataframe.rename(columns={0: "Дата", 1: "Трафик"}, inplace=True)
      dataframe['Трафик'] = [float(i.replace(',','')) for i in dataframe['Трафик']]
      print(dataframe[:5])
      

    elif kwargs['путь'] == 'вакансии':
      path = '/content/вакансии'
      assert os.path.exists(path), f'Загрузите базу \'{kwargs["путь"]}\'.'
      num = 3
      data = pd.read_csv(path + '/data.csv', index_col=0)

      columns=['Пол', 'Возраст', 'Город', 'Готовность к переезду',
              'Готовность к командировкам', 'Гражданство', 'Разрешение на работу',
              'Знания языков', 'Образование', 'Дополнительное образование',
              'Зарплата', 'Время в пути до работы', 'Занятость', 
              'График', 'Опыт работы (мес)', 'Обязанности на пред.работе']

      idx = np.random.randint(0, data.shape[0], num)
      for i in idx:
          print('Пример резюме:')
          print()

          for column in columns:
              info = str(data[column][i])
              info = info[:600]
              if info == 'no_data':
                  info = 'Данные не указаны'
              step = 100
              for j in range(0, len(info), step):
                  if j == 0:
                      print('%-28s' % (column + ':'), info[j:j+step])
                  else:
                      print('%-28s' % (''), info[j:j+step])
          print('---------------------------------------------------------------')
          result = data['Этап сделки'][i]
          print('%-28s' % 'Подходит ли кандидат: ', result)
          print('---------------------------------------------------------------')
          print()
    
    elif kwargs['путь'] not in new_data:
      if not 'размер' in kwargs:
          kwargs['размер'] = (108, 192)
      func_true_false(kwargs['путь'], type_check='dataset')
      if kwargs['путь']=='майонез':
          kwargs['размер'] = (192, 108)
      
      if kwargs['размер'] == 'молочная_продукция':
          kwargs['размер'] = (512,256)
      kwargs['путь'] = '/content/'+kwargs['путь']
      count_classes = sorted(os.listdir(kwargs['путь']))
      count = 5    
      fig, axs = plt.subplots(len(count_classes), count, figsize=(25, 15)) #Создаем полотно из 3 графиков
      for idx,num in enumerate(count_classes): #Проходим по всем классам
        car_path = os.path.join(kwargs['путь'], num)
        for i_1, curr_img in enumerate(sorted(os.listdir(car_path))[:count]):
            img_path = os.path.join(car_path,curr_img) #Выбираем случайное фото для отображения
            axs[idx, i_1].imshow(image.load_img(img_path, target_size=kwargs['размер'])) #Отображение фотографии
            axs[idx, i_1].axis('off') # отключаем оси
      plt.show() #Показываем изображения
    else:
        assert False, f'Невозможно проверить указанный путь: \'{kwargs["путь"]}\'.\
        Возможно вы имели ввиду \'{check_for_errors.check_word(kwargs["путь"], "show_paths")}\'.'  
        
  elif ('изображения' in kwargs):
    kwargs['изображения'] = kwargs['изображения'].lower()
    if 'метки' in kwargs:
      count = kwargs['метки'].max() # Задачем количество примеров
    elif 'количество' in kwargs:
      count = kwargs['количество'] # Задачем количество примеров
    else:
      count = 5
    f, axs = plt.subplots(1,count,figsize=(22,5)) # Создаем полотно для визуализации
    idx = np.random.choice(kwargs['изображения'].shape[0], count) # Получаем 5 случайных значений из диапазона от 0 до 60000 (x_train_org.shape[0])
    for i in range(count):
      axs[i].imshow(kwargs['изображения'][idx[i]], cmap='gray') # Выводим изображение из обучающей выборки в черно-белых тонах
      axs[i].axis('off')
    plt.show()
  elif ('оригиналы' in kwargs):
    kwargs['оригиналы'] = kwargs['оригиналы'].lower()
    показать_примеры_сегментации(**kwargs)
  elif ('изображения_низкого_качества' in kwargs):
    kwargs['изображения_низкого_качества'] = kwargs['изображения_низкого_качества'].lower()
    показать_примеры_HR(**kwargs)
    

def показать_примеры_HR(**kwargs):
  lr = kwargs['изображения_низкого_качества']
  hr = kwargs['изображения_высокого_качества']
  '''
  показать_примеры_изображений - функция вырезает небольшие кусочки из hr и lr изображений и выводит в масштабе на экран
  вход:
    lr, hr - списки с lr, hr изображениями соответственно
  '''
  n = 3 # Указываем кол-во отображаемых пар изображений
  fig, axs = plt.subplots(n, 2, figsize=(10, 15)) # Задаем размера вывода изображения

  for i in range(n): # В цикле попарно выводим изображения
    ind = random.randint(0, len(lr)) # Выбираем случайный индекс изображения

    area = (100, 100, 200, 200) # Задаем координаты точек для вырезания участка из изображения низкого качества
    cropped_lr = lr[ind].crop(area) # Вырезаем участок
    area = (400, 400, 800,800) # Задаем координаты точек для вырезания участка из изображения высокого качества
    cropped_hr = hr[ind].crop(area) # Вырезаем участок

    axs[i,0].axis('off')
    axs[i,0].imshow(cropped_lr) # Отображаем lr изображение
    axs[i,0].set_title('Низкое качество', fontsize=30)
    axs[i,1].axis('off')
    axs[i,1].imshow(cropped_hr) # Отображаем hr изображение
    axs[i,1].set_title('Высокое качество', fontsize=30)
  plt.show() #Показываем изображения

###
###                СОЗДАНИЕ ВЫБОРОК
###

def создать_выборки(**kwargs):
  путь = kwargs['путь']
  data_list_seg = ['самолеты','самолеты_макс','губы','люди']
  if not os.path.exists('/content/test_sets'):
    os.mkdir('/content/test_sets')
    
    
  if kwargs['путь'] == 'mnist':
      (x_train, y_train), (x_test, y_test) = datasets.mnist.load_data()
  
  elif kwargs['путь'] == 'трекер':
    (x_train, y_train), (x_test, y_test) = img_loader(**kwargs)
      
  elif kwargs['путь'] in ['tesla','симптомы_заболеваний']:
    return обработка_текста.создать_выборки(**kwargs)
  
  elif kwargs['путь']=='квартиры':
      return квартиры.создать_выборки()
  
  elif kwargs['путь']=='трейдинг':
      kwargs.pop('путь')
      (x_train, y_train), (x_test, y_test) = traide.получить_данные(**kwargs)
      
  elif kwargs['путь']=='умный_дом':
    kwargs.pop('путь')
    (x_train, y_train), (x_test, y_test) = home.создать_выборки(**kwargs)
  
  elif kwargs['путь']=='трафик':
    dataframe = pd.read_csv('traff.csv', header=None)
    dataframe.rename(columns={0: "Дата", 1: "Трафик"}, inplace=True)
    dataframe['Трафик'] = [float(i.replace(',','')) for i in dataframe['Трафик']]
    return dataframe
  
  elif kwargs['путь']=='вакансии':
    (x_train, y_train), (x_val, y_val) =модель.создать_выборки_вакансии()
    return (x_train, y_train), (x_val, y_val)
    
  elif kwargs['путь']=='диалоги':
    f = open('dialog.txt', 'r', encoding='utf-8')
    text= f.read()
    text = text.replace('"','')
    text = text.split('\n')
    question = text[::3]
    answers = text[1::3]
    return question[:-1], answers
    
  elif kwargs['путь'] == 'повышение_размерности':
      return формирование_повышение_размерности()
  
  elif kwargs['путь'] == 'договоры':
     return загрузить_договоры()

  elif kwargs['путь'] in data_list_seg:
      x, y = обработка_изображений(**kwargs)
      (x_train, y_train), (x_test, y_test) = сегментация.create_xy(x,y)
  else:
    путь_con = '/content/'+kwargs['путь']
    x_train = [] # Создаем пустой список, в который будем собирать примеры изображений обучающей выборки
    y_train = [] # Создаем пустой список, в который будем собирать правильные ответы (метки классов: 0 - Феррари, 1 - Мерседес, 2 - Рено)
    x_test = [] # Создаем пустой список, в который будем собирать примеры изображений тестовой выборки
    y_test = [] # Создаем пустой список, в который будем собирать правильные ответы (метки классов: 0 - Феррари, 1 - Мерседес, 2 - Рено)
    print('Создание наборов данных для обучения модели...')
    for j, d in enumerate(sorted(os.listdir(путь_con))):
        files = sorted(os.listdir(путь_con + '/'+d))    
        count = len(files) * (1-kwargs['коэф_разделения'])
        for i in range(len(files)):
            sample = image.load_img(путь_con + '/' +d +'/'+files[i], target_size=(kwargs['размер'])) # Загружаем картинку
            img_numpy = np.array(sample) # Преобразуем зображение в numpy-массив
            if i<count:
              x_train.append(img_numpy) # Добавляем в список x_train сформированные данные
              y_train.append(j) # Добавлеям в список y_train значение 0-го класса
            else:
              x_test.append(img_numpy) # Добавляем в список x_test сформированные данные
              y_test.append(j) # Добавлеям в список y_test значение 0-го класса
    display.clear_output(wait=True)
    x_train = np.array(x_train) # Преобразуем к numpy-массиву
    y_train = np.array(y_train) # Преобразуем к numpy-массиву
    x_test = np.array(x_test) # Преобразуем к numpy-массиву
    y_test = np.array(y_test) # Преобразуем к numpy-массиву
    x_train = x_train/255.
    x_test = x_test/255.
    print('Размер сформированного массива обучающая_выборка:', x_train.shape)
    print('Размер сформированного массива метки_обучающей_выборки:', y_train.shape)
    print('Размер сформированного массива тестовая_выборка:', x_test.shape)
    print('Размер сформированного массива метки_тестовой_выборки:', y_test.shape)
    
  np.save(f'/content/test_sets/{путь}.npy', x_test)
  np.save(f'/content/test_sets/{путь}_метки.npy', y_test)

  return (x_train, y_train), (x_test, y_test)

def img_loader(путь='трекер', размер=(64,64), коэф_разделения=0.2):
    
    """
    Parameters:
    путь (str)              Путь к папке с датасетом
    размер (tuple)           Размер изображения

    Return:
    x_train, y_train, x_val, y_val
    """
    путь = '/content/'+ путь
    heads = pd.read_csv('/content/трекер/Heads.csv', index_col=0)

    data = []
    labels = []

    for i in range(heads.shape[0]):
        # указываем пути к очередной паре картинок
        first_path = os.path.join(путь, 'images', heads['First image'][i])
        second_path = os.path.join(путь, 'images', heads['Second image'][i])
        
        # загрузка и преобразование к нужному размеру первого изображения
        first_img = Image.open(first_path)
        first_img.thumbnail(размер)
        first_img = np.array(first_img)

        # загрузка и преобразование к нужному размеру второго изображения
        second_img = Image.open(second_path)
        second_img.thumbnail(размер)
        second_img = np.array(second_img)

        # склеиваем по каналам картинки в один массив
        concat_img = np.concatenate((first_img, second_img), axis=2)

        data.append(concat_img)

        label = heads['Label'][i]
        labels.append(label)
    
    data = np.array(data) / 255.0
    labels = np.array(labels)

    x_train, x_val, y_train, y_val = train_test_split(data, labels, test_size = коэф_разделения, shuffle = True)
    print('Размер сформированного массива обучающая_выборка:', x_train.shape)
    print('Размер сформированного массива метки_обучающей_выборки:', y_train.shape)
    print('Размер сформированного массива тестовая_выборка:', x_val.shape)
    print('Размер сформированного массива метки_тестовой_выборки:', y_val.shape)
    return (x_train, y_train), (x_val, y_val)

def создать_выборки_вакансии():
  global idx_train, idx_val
  dir = '/content/вакансии'
    
  """
  Parameters:
  dir (str)              Путь к папке с датасетом

  Return:
  x_train, y_train, x_val, y_val
  """

  data = np.load(dir + '/X_train.npy')
  labels = np.load(dir + '/Y_train.npy')

  x_train, x_val, y_train, y_val = data[:500], data[500:], labels[:500], labels[500:]
  idx_val = [i for i in range(data[500:].shape[0])]

  return (x_train, y_train), (x_val, y_val)

def предобработка_данных(**kwargs):
  if kwargs['сетка'] == 'полносвязная':
    x_train = kwargs['изображения']/255. # Нормируем изображения, приводя каждое значение пикселя к диапазону 0..1
    x_train = x_train.reshape((-1, 28*28)) # Изменяем размер изображения, разворачивая в один вектор
    print('Размер сформированных данных:', x_train[0].shape) # Выводим размер исходного изображения
    return x_train
  elif kwargs['сетка'] == 'сверточная':
    x_train = kwargs['изображения']/255. # Нормируем изображения, приводя каждое значение пикселя к диапазону 0..1
    x_train = x_train.reshape((-1, 28,28,1)) # Изменяем размер изображения, разворачивая в один вектор
    print('Размер сформированных данных:', x_train[0].shape) # Выводим размер исходного изображения
    return x_train

def обработка_изображений(**kwargs):
  return сегментация.get_images(**kwargs)

def распаковать_архив(откуда='', куда=''):
  proc = subprocess.Popen('unzip -q "' + откуда + '" -d ' + куда, shell=True, stdin=None, stdout=open(os.devnull,"wb"), stderr=STDOUT, executable="/bin/bash")
  proc.wait()

def показать_примеры_сегментации(**kwargs):
  сегментация.show_sample(**kwargs)

def загрузить_базу_ОДЕЖДА():
  (x_train_org, y_train_org), (x_test_org, y_test_org) = datasets.fashion_mnist.load_data() # Загружаем данные набора MNIST
  display.clear_output(wait=True)
  print('Данные загружены')
  print('Размер обучающей выборки:', x_train_org.shape) # Отобразим размер обучающей выборки
  print('Размер тестовой выборки:', x_test_org.shape) # Отобразим размер тестовой выборки
  return (x_train_org, y_train_org), (x_test_org, y_test_org)

def загрузить_базу_СТРОЙКА(**kwargs):
  global url
  if 'путь' in kwargs:
	  path = '/content/drive/MyDrive/' + kwargs['путь'] + 'AIU.zip'
  else:
	  path = '/content/drive/MyDrive/AIU.zip'
  print('Загрузка данных')
  print('Это может занять несколько минут...')
  распаковать_архив(
    откуда = '/content/drive/MyDrive/AIU.zip',
    куда = '/content'
  )
  распаковать_архив(
      откуда = "Notebooks.zip",
      куда = "/content"
  )
  x_train = np.load('xTrain_st.npy')
  x_test = np.load('xVal_st.npy')
  y_train = np.load('yTrain_st.npy')
  y_test = np.load('yVal_st.npy')  
  print('Загрузка данных (Готово)')
  return (x_train, y_train), (x_test, y_test)
  
  print('Загрузка данных...')
  urls = ['https://drive.google.com/uc?export=download&confirm=no_antivirus&id=1WtUbopKzQw97W8DChDu0JidJYh08XQyy',
            'https://drive.google.com/uc?export=download&confirm=no_antivirus&id=14gsGpYv13IMUKXmjQEPhPt2bVpVkcJfY',
            'https://drive.google.com/uc?export=download&confirm=no_antivirus&id=1A9IThR5f7dUIHgohDDJBeFyAZquTiYIL',
            'https://drive.google.com/uc?export=download&confirm=no_antivirus&id=1PtMhqaPXYoJKuKjLy338rBgv-PsScYPh',
            'https://drive.google.com/uc?export=download&confirm=no_antivirus&id=1qRAPeOgCZ0g9nikKmop4uWGEe9cCSm4B',
            'https://drive.google.com/uc?export=download&confirm=no_antivirus&id=1MDJiPs1Lyh-ij5dldjAu-kwWWb2uPNKi',
            'https://drive.google.com/uc?export=download&confirm=no_antivirus&id=1S7bc5yR2fHsR81aDZsIpBcCfxf-43Cek'
            '']
  for url in urls:    
    output = 'data.zip' # Указываем имя файла, в который сохраняем файл
    gdown.download(url, output, quiet=True) # Скачиваем файл по указанному URL
    if os.path.exists('data.zip'):
        break  
  # Скачиваем и распаковываем архив
  распаковать_архив(
      откуда = "data.zip",
      куда = "/content"
  )
  x_train = np.load('xTrain_st.npy')
  x_test = np.load('xVal_st.npy')
  y_train = np.load('yTrain_st.npy')
  y_test = np.load('yVal_st.npy')  
  print('Загрузка данных (Готово)')
  return (x_train, y_train), (x_test, y_test)


def func_true_false(name, type_check = "show_paths"):
    name = name.replace('/','')
    curr = check_for_errors.check_word(name, type_check)
    name_check = (curr==name)
    path_check = os.path.exists('/content/'+curr)
    if name == 'диалоги':
        path_check = os.path.exists(os.path.join('/content/','dialog.txt'))
    elif name == 'трафик':
        path_check = os.path.exists(os.path.join('/content/','traff.csv'))
    elif name == 'квартиры':
        path_check = os.path.exists(os.path.join('/content/','moscow.csv'))
    elif name_check:
        if path_check:
            pass
        else:
            assert False, f'Загрузите базу \'{curr}\'.'
    else:
        if path_check:
            assert False, f'Ваше название \'{name}\' неверное. Возможно вы имели ввиду \'{curr}\'.'
        else:
            assert False, f'Перепроверьте название и загрузите вашу базу. Работайте для базы \'{curr}\'.'


from . import  терра_ии
терра_ии.выполнить_команду('pip install python-Levenshtein') 
import Levenshtein
import numpy as np
import re
from IPython import display
from IPython.display import clear_output
from tqdm.notebook import tqdm_notebook
import time
import os
import requests
import datetime
import time
import requests
from termcolor import colored
display.clear_output(wait=True)



datasets = ['mnist','авто-2','авто-3', 'молочная_продукция','автобусы', 'tesla', 'обнаружение_возгораний', 'майонез', 'трекер',\
             'вакансии', 'самолеты', 'самолеты_макс', 'люди', 'кожные_заболевания', 'повышение_размерности', 'обнаружение_людей',\
             'симптомы_заболеваний', 'писатели', 'диалоги', 'договоры', 'квартиры', 'губы', 'трафик', 'умный_дом', 'трейдинг']

all_layers = ['Полносвязный','Повтор','Эмбеддинг','Сверточный2D','Сверточный1D','Выравнивающий',\
          'Нормализация','Нормализация_01','Нормализация_11','Денормализация','Активация',\
          'ЛСТМ','МаксПуллинг2D','МаксПуллинг1D','Дропаут','PReLU','LeakyReLU']

all_task = ['классификация изображений', 'классификация вакансий','временной ряд','аудио',\
             'сегментация изображений','сегментация текста']
             
all_trade = ['газпром', 'яндекс', 'полиметаллы']

all_home =['кондиционер','свет','телевизор','фон']

all_paths = ['молочная_продукция', 'автобусы', 'майонез', 'обнаружение_возгораний',
           'авто-2','авто-3','молочная_продукция','умный_дом', 'договоры', 'tesla',\
           'симптомы_заболеваний','писатели', 'трекер', 'вакансии', 'mnist']

def check_word(arg, word_type=None, extra=None):
    arg = arg.upper()
    if extra:
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in extra]
        return f'{extra[np.argmin(distances)]}' 
    if word_type=='dataset':
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in datasets]
        return f'{datasets[np.argmin(distances)]}'
    elif word_type=='layer':
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in all_layers]
        return f'{all_layers[np.argmin(distances)]}'
    elif word_type=='task':
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in all_task]
        return f'{all_task[np.argmin(distances)]}' 
    elif word_type=='trade':
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in all_trade]
        return f'{all_trade[np.argmin(distances)]}'
    elif word_type=='home':
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in all_home]
        return f'{all_home[np.argmin(distances)]}'
    elif word_type=='show_paths':
        distances = [Levenshtein.distance(arg.upper(), i.upper()) for i in all_paths]
        return f'{all_paths[np.argmin(distances)]}' 


def decorator_check(func):
  def inner_func(arg, **kwargs):
    try:
      return func(arg, **kwargs)
    except ValueError:
      assert False, f'У вас присутсвует строка \'{arg}\', проверьте наличие отсупа или пробела в этом месте.'
  return inner_func

def test_homework(arg):
    check_words = []
    if arg==1:
        paths = [os.path.exists('/content/car.zip'), os.path.exists('/content/mayonnaise.zip')]
    elif arg==2:
        paths = [os.path.exists('/content/HR.zip'), os.path.exists('/content/tesla.zip'), os.path.exists('/content/symptoms.zip')]
    elif arg==3:
        paths = [os.path.exists('/content/cHome.zip'), os.path.exists('/content/shares.zip'), os.path.exists('/content/lips.zip')]
    else:
        assert False,'Введите число от 1 до 3, где число отвечает за номер выполненого домашнего задания.'
    for i in range(1,len(paths)+1):
        if paths[i-1]:
            pass
        else:
            check_words.append(f'Вы ещё не работали с задачей-{i}\n')
    for i in tqdm_notebook(range(len(paths) - len(check_words)), desc='HW', total = len(paths)):
        time.sleep(0.5)

    for i in check_words:
        print(i)

    if len(check_words)==0:
        return True
    return False

def send_work(dict_arg_homework = {1:1441, 2:1442, 3:1446}, LP = 204886, online='ваш учебный день', home='интенсив'):
    numbers = list(dict_arg_homework)
    numbers_1 = ''
    for i in numbers[:-1]:
        numbers_1+=f'{i} '
    education_number = int(input(f'Введите {online} - {numbers_1}или {numbers[-1]}: '))
    display.clear_output()
    if home=='интенсив':
        result = test_homework(education_number)
    else:
        pass
    if result:
        username = input('Введите ваш email: ')
        response = requests.get('https://skochnev.dev.neural-university.com/api/v2/users',
                                params ={'username': f'{username}'})
        if response.json()['hydra:member']:
            id_user = response.json()['hydra:member'][0]['id']
            time_now = str(datetime.datetime.fromtimestamp(time.time()+10800))
            params = {
            "homework": f"/api/v2/homework/{dict_arg_homework[education_number]}",
            "description": "Ваша работа зачтена. Успехов!",
            "filing_at": str(time_now).replace(' ','T')+'Z',
            "student": f"/api/v2/users/{id_user}",
            "canComment": True,
            "vision": 0,
            "points": 1,
            "status": 4,
            "learningProgramm": f"/api/v2/learning_programms/{LP}",
            'complexity_level':1
            }
            response = requests.post(
                'https://skochnev.dev.neural-university.com/api/v2/homework_results',
                json=params)
            print('Вы сдали!')
        else:
            assert False, 'Данный email не был найден в базе.'
    

def func_dict(kwargs, dict_vals):
    for i in dict_vals:
        if i in kwargs:
            dict_vals[i] = func_is_digit(kwargs[i])
        else:
            pass
    return dict_vals
    
    
def func_is_digit(arg):
    if isinstance(arg, tuple):
        return arg
    elif isinstance(arg,str): 
        if arg.isdigit():
            return int(arg)
        try:
            return float(arg)
        except ValueError:
            return arg
    return arg 


def decorator_check_hw(func):
  def inner_func(**kwargs):
    try:
      return func(**kwargs)
    except ValueError:
      assert False, f'Убедитесь, что все ваши нейронки имеют разные названия. В случае данной ошибки сбросьте ноутбук и запустите ячейки по новой с разными названиями нейронок.'
    except KeyError as kr:
        assert False, f'Нейронки {kr} нет. Перепроверьте название.' 
    except FileNotFoundError as err:
        assert False, f'Загрузите выборки для базы  \'{str(err)[57:-5]}\'.'
  return inner_func



@decorator_check_hw
def check_hw(**kwargs):
    all_mistakes = []
    for i in range(len(kwargs['datasets'])):
        if isinstance(kwargs[f'нейронка_{i+1}'].input_shape, tuple):
            len_shape = 1
        elif isinstance(kwargs[f'нейронка_{i+1}'].input_shape, list):
            len_shape = len(kwargs[f'нейронка_{i+1}'].input_shape)
        if len_shape == 1:
            x_test = np.load(f'/content/test_sets/{kwargs["datasets"][i]}.npy')
        elif len_shape == 3:
            x_test = []
            for num in range(3):
                curr_x_test = np.load(f'/content/test_sets/{kwargs["datasets"][i]}{num}.npy')
                x_test.append(curr_x_test)
        
        y_test = np.load(f'/content/test_sets/{kwargs["datasets"][i]}_метки.npy')
        accur = kwargs[f'нейронка_{i+1}'].evaluate(x_test, y_test, verbose=0)[1]
        if accur>kwargs["accuracies"][i]:
            pass
        else:
            all_mistakes.append(f'Точность {i+1}-ой задачи - {round(accur*100,2)}%, когда требовалось минимально {100*kwargs["accuracies"][i]}%')
    return all_mistakes

def test_hw(**kwargs):
    accuracies = [[0.6, 0.85],[0.1,0.1,0.1],[0.1,0.1,0.1]]
    datasets = [['авто-3', 'майонез'], ['вакансии', 'tesla', 'симптомы_заболеваний'],['умный_дом','трейдинг', 'губы']]
    temporary_var = {'datasets':datasets[kwargs['lesson']],'accuracies':accuracies[kwargs['lesson']]}
    kwargs.update(temporary_var)
    result = check_hw(**kwargs)
    clear_output()
    for i in tqdm_notebook(range(len(accuracies[kwargs['lesson']]) - len(result)), desc='HW', total = len(accuracies[kwargs['lesson']])):
        time.sleep(0.5)
    for i in result:
        print(i)

    if len(result)==0:
        return True
    return False
    




def post_homework(username, hw_id):

  # Зачет ДЗ
  homework_id = hw_id
  curator_id = 93854
  r = requests.post('https://aiu-hw-provider.herokuapp.com/api/', data={'email': f'{username}', 'homework_id': f'{homework_id}'})
  return r.json()['homework_assignment']
    

def status_message(res):
    if res['answer'] == -1:
        raise NameError('Указанный пользователь не найден.')
    elif res['answer'] == -2:
        print('Домашняя работа уже принята.')
    elif res['answer'] == -3:
        raise Exception('Непредвиденная ошибка, обратитесь к куратору.')
    elif res['answer'] == 1:
        print('Всё верно! Работа принята.')


def test_hw_1(**kwargs):
    homework_id = 1633    
    kwargs.update({'lesson':0})
    if test_hw(**kwargs):
        res = post_homework(kwargs['логин'], homework_id)
        status_message(res)
    
def test_hw_2(**kwargs):
    homework_id = 1634    
    kwargs.update({'lesson':1})
    if test_hw(**kwargs):
        res = post_homework(kwargs['логин'], homework_id)
        status_message(res)
    
def test_hw_3(**kwargs):
    homework_id = 1635    
    kwargs.update({'lesson':2})
    if test_hw(**kwargs):
        res = post_homework(kwargs['логин'], homework_id)
        status_message(res)







from . import сегментация, повышение_размерности, датасет, check_for_errors
from tensorflow.keras.preprocessing.text import tokenizer_from_json
from tensorflow.keras.layers import *
from tensorflow.keras.models import load_model, Sequential, Model # Подключаем модель типа Sequential
from tensorflow.keras.optimizers import Adam
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
from tensorflow.keras.utils import to_categorical, plot_model # Полкючаем методы .to_categorical() и .plot_model()
sns.set_style('darkgrid') 
from tensorflow.keras import backend as K # Импортируем модуль backend keras'а
from tensorflow.keras.optimizers import Nadam, RMSprop, Adadelta,Adam # Импортируем оптимизатор Adam
from tensorflow.keras.callbacks import ModelCheckpoint, LambdaCallback, ReduceLROnPlateau
from tensorflow.keras.layers import Input, RepeatVector, Conv2DTranspose, concatenate, Activation, Embedding, Input, MaxPooling2D, Conv2D, BatchNormalization # Импортируем стандартные слои keras
import importlib.util, sys, gdown
import tensorflow as tf
from PIL import Image
import pandas as pd
from IPython.display import clear_output
from tensorflow.keras.preprocessing import image
import termcolor
from termcolor import colored
from google.colab import files
import subprocess, os, warnings, time
from pandas.core.common import SettingWithCopyWarning
from subprocess import STDOUT, check_call
import requests
import random
import zipfile
from sklearn.model_selection import train_test_split
import ast
from tabulate import tabulate
import getpass
from tensorflow.keras.datasets import mnist
from tensorflow.keras.layers import *
from tensorflow.keras.models import *
warnings.filterwarnings('ignore')
import logging
tf.get_logger().setLevel(logging.ERROR)

import nltk, pymorphy2, io, json
from tensorflow.keras.preprocessing.text import Tokenizer, text_to_word_sequence
from keras.layers import *
from tensorflow.keras.preprocessing.sequence import pad_sequences
nltk.download('stopwords')


def создать_выборки_вакансии_test():
  global idx_val
  dir = '/content/Вакансии'
  """
  Parameters:
  dir (str)              Путь к папке с датасетом

  Return:
  x_train, y_train, x_val, y_val
  """

  data = np.load(dir + '/X_train.npy')
  labels = np.load(dir + '/Y_train.npy')

  x_val, y_val = data[300:], labels[300:]
  idx_val = [i for i in range(data[300:500].shape[0])]

  return (x_train, y_train), (x_val, y_val)



    
def создать_выборки_test(maxWordsCount, xLen, step, путь_к_базе='content/Болезни/'):
  path = путь_к_базе
  text = []
  classes = []
  n = 0
  codecs_list = ['UTF-8', 'Windows-1251']

  for filename in sorted(os.listdir(path)): # Проходим по всем файлам в директории договоров
      n +=1
      for codec_s in codecs_list:
        try:
            text.append(readText(path+filename, codec_s)) # Преобразуем файл в одну строку и добавляем в agreements
            classes.append(filename.replace(".txt", ""))
            break
        except UnicodeDecodeError:
            print('Не прочитался файл: ', path+currdir+'/'+filename, codec_s)
        else:
            next 

  stop_words = nltk.corpus.stopwords.words('russian')
  lexeme_list = ['POS', 'animacy', 'aspect', 'case', 'gender', 'involvement', 'mood', 'number', 'person', 'tense', 'transitivity', 'voice']

  words = [] # Здесь будут лежать все списки слов каждого из описаний заболеваний
  tags = []   # Здесь будут лежать все списки списков граммем для каждого слова
  tags_all = [] # Здесь будут лежать все списки граммем всех слов для тренировки токенайзера
  for i in range(len(text)):
    word, tag = text2Words(text[i])
    words.append(word)
    tags.append(tag)
  for k in tags:
    for t in k:
      tags_all.append(t)

  tokenizer = Tokenizer(num_words=maxWordsCount, filters='!"#$%&()*+,-––—./:;<=>?@[\\]^_`{|}~\t\n\xa0', lower=True, split=' ', oov_token='unknown', char_level=False)
  tokenizer.fit_on_texts(words) 
  items = list(tokenizer.word_index.items())


  tokenizer_json1 = tokenizer.to_json()
  with io.open('tokenizer1.json', 'w', encoding='utf-8') as f:
      f.write(json.dumps(tokenizer_json1, ensure_ascii=False))
  with open('tokenizer1.json') as f:
      data = json.load(f)
      tokenizer = tokenizer_from_json(data)

  items = list(tokenizer.word_index.items()) 

  maxWordsCount2 = 50 

  tokenizer2 = Tokenizer(num_words=maxWordsCount2, filters='!"#$%&()*+,-––—./:;<=>?@[\\]^_`{|}~\t\n\xa0', lower=True, split=' ', oov_token='unknown', char_level=False)

  tokenizer2.fit_on_texts(tags_all) 
  items2 = list(tokenizer2.word_index.items()) 

  tokenizer_json2 = tokenizer2.to_json()
  with io.open('tokenizer2.json', 'w', encoding='utf-8') as f:
      f.write(json.dumps(tokenizer_json2, ensure_ascii=False))

  with open('tokenizer2.json') as f:
      data = json.load(f)
      tokenizer2 = tokenizer_from_json(data)

  items2 = list(tokenizer2.word_index.items())

  xTrainIndexes = tokenizer.texts_to_sequences(words) 
  xTrainTagsIndexes = []
  for tag in tags:
    xTrainTagsIndexes.append(tokenizer2.texts_to_sequences(tag))
  nVal = 200   
  valWords = []    
  valTagsWords = []  
  for i in range(len(xTrainIndexes)):
    valWords.append(xTrainIndexes[i][-nVal:])
  for i in range(len(xTrainTagsIndexes)):
    valTagsWords.append(xTrainTagsIndexes[i][-nVal:])

  (xVal, yVal) = createSetsMultiClasses(valWords, xLen, step)
  (xTagsVal, _) = createSetsMultiClasses(valTagsWords, xLen, step)

  xVal01 = tokenizer.sequences_to_matrix(xVal.tolist(), mode="tfidf")
  xTagsVal = np.reshape(xTagsVal, (xTagsVal.shape[0], -1))
  xTagsVal01 = tokenizer2.sequences_to_matrix(xTagsVal.tolist())


  x_val = [xVal, xVal01, xTagsVal01]
  y_val = yVal
  clear_output(wait=True)
  print('Формирование выборки завершено')
  return x_val, y_val

def readText(fileName, encod):
    f = open(fileName, 'r', encoding=encod)
    text = f.read()
    text = text.replace("\n", " ")
    return text

def text2Words(text):
  stop_words = nltk.corpus.stopwords.words('russian')
  lexeme_list = ['POS', 'animacy', 'aspect', 'case', 'gender', 'involvement', 'mood', 'number', 'person', 'tense', 'transitivity', 'voice']

  text = text.replace(".", " ")
  text = text.replace("—", " ")
  text = text.replace(",", " ")
  text = text.replace("!", " ")
  text = text.replace("?", " ")
  text = text.replace("…", " ")
  text = text.replace("-", " ")
  text = text.replace("(", " ")
  text = text.replace(")", " ")
  text = text.replace(";", " ")
  text = text.replace("°c", " ")
  text = text.replace("–", " ")
  text = text.replace("и/ить", " ")
  text = text.lower()
  morph = pymorphy2.MorphAnalyzer()
  
  words = []
  tags = []
  currWord = ""
  for symbol in text[1:]:

    if (symbol != " "):
      currWord += symbol
    else:
      if (currWord != "") & (currWord not in stop_words): # 
        words.append(currWord)
        currWord = ""
      else:
        currWord = ""
  if (currWord != "") & (currWord not in stop_words): #
        words.append(currWord)
  # генератором списков проходимся по каждому сформированному слову и лемматиризируем его с помощью pymorphy до получения граммем
  #words = [morph.parse(word)[0].normal_form for word in words]
  #tags = [morph.parse(word)[0].tag for word in words] # .cyr_repr
  for word in words:
    tag_word = []
    lex_0 = morph.parse(word)[0].tag.POS  # Part of Speech, часть речи
    if lex_0 != None:
        tag_word.append(lex_0)
    else:
        tag_word.append('not')
    lex_1 = morph.parse(word)[0].tag.animacy  # одушевленность
    if lex_1 != None:
        tag_word.append(lex_1)
    else:
        tag_word.append('not')
    lex_2 = morph.parse(word)[0].tag.aspect # вид: совершенный или несовершенный
    if lex_2 != None:
        tag_word.append(lex_2)
    else:
        tag_word.append('not')
    lex_3 = morph.parse(word)[0].tag.case # падеж
    if lex_3 != None:
        tag_word.append(lex_3)  
    else:
        tag_word.append('not')     
    lex_4 = morph.parse(word)[0].tag.gender # род (мужской, женский, средний)
    if lex_4 != None:
        tag_word.append(lex_4)
    else:
        tag_word.append('not') 
    lex_5 = morph.parse(word)[0].tag.involvement  # включенность говорящего в действие
    if lex_5 != None:
        tag_word.append(lex_5)
    else:
        tag_word.append('not')
    lex_6 = morph.parse(word)[0].tag.mood # наклонение (повелительное, изъявительное)
    if lex_6 != None:
        tag_word.append(lex_6)
    else:
        tag_word.append('not')
    lex_7 = morph.parse(word)[0].tag.number # число (единственное, множественное)
    if lex_7 != None:
        tag_word.append(lex_7)
    else:
        tag_word.append('not')
    lex_8 = morph.parse(word)[0].tag.person # лицо (1, 2, 3)
    if lex_8 != None:
        tag_word.append(lex_8)
    else:
        tag_word.append('not')
    lex_9 = morph.parse(word)[0].tag.tense  # время (настоящее, прошедшее, будущее)
    if lex_9 != None:
        tag_word.append(lex_9)
    else:
        tag_word.append('not')
    lex_10 = morph.parse(word)[0].tag.transitivity  # переходность (переходный, непереходный)
    if lex_10 != None:
        tag_word.append(lex_10)
    else:
        tag_word.append('not')
    lex_11 = morph.parse(word)[0].tag.voice # залог (действительный, страдательный)
    if lex_11 != None:
        tag_word.append(lex_11)
    else:
        tag_word.append('not')
    tags.append(tag_word)
    # генератором списков проходимся по каждому сформированному слову и лемматиризируем его с помощью pymorphy
  words = [morph.parse(word)[0].normal_form for word in words]

  return words, tags



import os, librosa
import numpy as np
import IPython.display as ipd # Воспроизведение звуковыйх файлов
from IPython import display
import librosa.display
from matplotlib import pyplot as plt
import time
from tqdm.notebook import tqdm_notebook as tqdm_
from sklearn.model_selection import train_test_split
from tensorflow.keras.utils import to_categorical
from matplotlib.patches import Rectangle
import librosa #Для параметризации аудио
import sklearn
from sklearn.model_selection import train_test_split #Разбиение на обучающую и проверочную выборку
from sklearn.preprocessing import LabelEncoder, StandardScaler #Для нормировки данных
from . import check_for_errors

def показать_примеры_голосовых_команд():
  i_start = 0.5
  i_end = 2.5

  список_категорий = ['Кондиционер', 'Свет', 'Телевизор', 'Фон']
  assert os.path.exists('/content/умный_дом/comands/'), 'Загрузите базу \'Умный дом\'.'
  директории = sorted(os.listdir('/content/умный_дом/comands/'))
  for i in range(len(список_категорий)):
    print ('Пример голосовой команды «', список_категорий[i],'»:',sep='')
    список_примеров = os.listdir('/content/умный_дом/comands/'+ директории[i])
    случайный_пример = список_примеров[np.random.randint(0, len(список_примеров[i]))]
    y,sr = librosa.load('/content/умный_дом/comands/'+ директории[i]+'/'+случайный_пример)
    ipd.display(display.Audio(data=y, rate = sr))    
    print ('Спектр голосовой команды:')  
    X = librosa.stft(y)
    Xdb = librosa.amplitude_to_db(abs(X))      

    plt.figure(figsize=(10, 6))
    librosa.display.specshow(Xdb, sr=sr, x_axis='time', y_axis='hz')
    plt.colorbar()
    plt.show()
    print()
    plt.figure(figsize=(30, 5))
    librosa.display.waveplot(y)
    plt.show()
    print()
    print()

def создать_выборки(длина, шаг):
  время_старта = time.time()

  # все константы
  тренировочная_путь = "/content/умный_дом/comands/" 
  length=11025
  sample_rate = 22050                   # Значение sample_rate аудиофайлов
  feature_dim_1 = 20                    # стандартная величина MFCC признаков
  feature_dim_2 = int(длина * sample_rate) # установленная длина фреймов (в секундах 0.5 = 500 мс)
  step_mfcc = int(шаг * sample_rate)    # Шаг смещения при разборе mfcc (в секундах 0.1 = 100
  split_ratio=0.8                       # split_ratio - (1-split_ratio) равно доле тестовых образцов, которые вернет функция train_test_split
  random_state=42                       # random_state - начальное число, используемое генератором случайных чисел в функции train_test_split

  labels = sorted(os.listdir(тренировочная_путь)) # запишем лейблы классов по названию папок с данными - ['кондиционер', 'телевизор', 'свет', 'фон']  
  label_indices = np.arange(0, len(labels))       # запишем лейблы в виде индексов - [0, 1, 2, 3]

  def wav2mfcc(file_path, length = 11025, step = 2205): 
    out_mfcc = []                   # Выходной массив, содержащий mfcc исходного файла с шагом step
    out_audio = []                  # Выходной массив, содеражищий аудиоинформацию исходного файла с шагом step
    y, sr = librosa.load(file_path) # Загружаем данные исходного файла  

    while (len(y)>=length):                               # Проходим весь массив y, пока оставшийся кусочек не станет меньше указанной в параметре max_len длинны
      section = y[:length]                                # Берем начальный кусок длинной length
      section = np.array(section)                         # Переводим в numpy
      out_mfcc.append(librosa.feature.mfcc(section, sr))  # Добавляем в выходной массив out_mfcc значение mfcc текущего куска
      out_audio.append(section)                           # Добавляем в выходной массив аудио текущий кусок
      y = y[step:]                                        # Уменьшаем y на step
    
    out_mfcc = np.array(out_mfcc)     # Преобразуем в numpy
    out_audio = np.array (out_audio)  # Преобразуем в numpy
    return out_mfcc, out_audio        # функция вернет массив мэл-частот и массив аудио-отрезков

  '''
  Объявим функцию формирования и сохранения векторов данных, полученных для каждого набора аудио-команд в датасете
  Параметры:
    path  - путь к папке, в которой находятся каталоги с обучающими командами    
    length - длина отрезков, на которые разбиваем исходный файл
  '''
  
  def save_data_to_array(path=тренировочная_путь, length=11025):
    for label in labels:    # для каждого лейбла
      mfcc_vectors = []     # здесь соберем векторы MFCC частот
      # извлечем для каждого файла его путь c названием папки и именем файла и соберём в список
      # ['/content/drive/My Drive/Речевые команды/свет/recording12.wav', '/content/drive/My Drive/Речевые команды/свет/recording11.wav'..]
      wavfiles = [path + label + '/' + wavfile for wavfile in os.listdir(path + '/' + label)]
      for wavfile in tqdm_(wavfiles, desc=('Сохраняем векторы класса ' + label), ncols=1000):
        mfcc, _ = wav2mfcc(wavfile, length=length, step = step_mfcc)                  # получим мел-частоты      
        if (mfcc.shape[0] != 0 ):                                                     # Если массив не нулевой длинны
          mfcc_vectors.extend(mfcc)                                                   # и добавим вектор в список для соответствующего класса
      np.save(label + '.npy', mfcc_vectors)                                           # сохраним массивы данных для каждого класса
  
  save_data_to_array(length=feature_dim_2)
   
  # подготовка выборок
  # откусываем первый кусок
  X = np.load('/content/'+labels[0] + '.npy')   # берем набор векторов для первого класса 
  y = np.zeros(X.shape[0], dtype = 'int32')     # устанавливаем размер соответствующего ему лейбла

  # Объединяем в единый датасет в виде np-массива обучающий и проверочный набор данных
  for i, label in enumerate(labels[1:]): 
    x = np.load('/content/'+label + '.npy')                     # Читаем очередной массив данных 
    X = np.vstack((X, x))                                       # Соединяем с исходным набором
    y = np.append(y, np.full(x.shape[0], fill_value= (i + 1)))  # В массив y добавлем x.shape[0]-элеменентов со значением (i + 1)

  # Формируем обучающую и проверочную выборки
  x_train, x_test, y_train, y_test = train_test_split(X, y, test_size= (1-split_ratio), random_state=random_state, shuffle=True)
  
  # Добавляем значение канала(=1) в размеры 'x' выборок
  x_train = x_train[..., None]
  x_test = x_test[..., None]

  y_train = to_categorical(y_train)   # представляем лейблы классов обучающего набора в виде one-hot вектора ((0, 1, 0) и т.п)
  y_test = to_categorical(y_test)     # представляем лейблы классов проверочного набора в виде one-hot вектора ((0, 1, 0) и т.п)

  число_аудио = 0
  for i in labels:
    число_аудио += len(os.listdir(тренировочная_путь+i))
  print()
  print('Данные успешно загружены. Обработано: ' + str(число_аудио) + ' аудио')
  print()
  время_на_обработку = time.time() - время_старта
  print('Время обработки: '+ str(int(время_на_обработку//60)) + ' мин '+ str(round(время_на_обработку%60,2)) +' c')
  return (x_train, y_train), (x_test, y_test)

def тест_модели(модель, порог, длина, шаг):
  sample_rate = 22050                   # Значение sample_rate аудиофайлов
  feature_dim_2 = int(длина * sample_rate) # установленная длина фреймов (в секундах 0.5 = 500 мс)
  step_mfcc = int(шаг/2 * sample_rate)    # Шаг смещения при разборе mfcc (в секундах 0.1 = 100
  classes = ['КОНДИЦИОНЕР', 'СВЕТ', 'ТЕЛЕВИЗОР']
  n_classes = 4

  #  функция параметризации аудио(wav в мел-частоты)
  def wav2mfcc(file_path, length = 11025, step = 2205): 
    out_mfcc = []                   # Выходной массив, содержащий mfcc исходного файла с шагом step
    out_audio = []                  # Выходной массив, содеражищий аудиоинформацию исходного файла с шагом step
    y, sr = librosa.load(file_path) # Загружаем данные исходного файла  

    while (len(y)>=length):                               # Проходим весь массив y, пока оставшийся кусочек не станет меньше указанной в параметре max_len длинны
      section = y[:length]                                # Берем начальный кусок длинной length
      section = np.array(section)                         # Переводим в numpy
      out_mfcc.append(librosa.feature.mfcc(section, sr))  # Добавляем в выходной массив out_mfcc значение mfcc текущего куска
      out_audio.append(section)                           # Добавляем в выходной массив аудио текущий кусок
      y = y[step:]                                        # Уменьшаем y на step
    
    out_mfcc = np.array(out_mfcc)       # Преобразуем в numpy
    out_audio = np.array (out_audio)    # Преобразуем в numpy
    return out_mfcc, out_audio          # функция вернет массив мэл-частот и массив аудио-отрезков

  # Объявим функцию предсказания команды
  def predict(namefile, model, min_count = 2, rate = порог, hole = 1):                    # функция принимает на вход путь к нужному файлу, и имя обученной модели 
    mfcc_full, audio_full = wav2mfcc(namefile, length=feature_dim_2, step = step_mfcc)  # Получаем массив mfcc выбранного файла с именем namefile    

    #mfcc = xScaler.transform(mfcc_full.reshape(-1,1))
    mfcc_full = mfcc_full.reshape(-1, 20, 22, 1)
    g_pred = model.predict(mfcc_full)               # Предиктим с помощью модели model массив mfcc
    pred = np.array([np.argmax(i) for i in g_pred]) # Выбираем индекс максимального элемента в каждом g_pred[i]  и создаем numpy-массив pred из этих индексов

    out = []    # Объявляем выходную переменную out (В ней будут храниться преобразованные из mfcc ауидоданные, класс команды и точность, с которой сеть считает эту команду верной)  
  
    # Ищем команды каждого класса
    for idx_class in range(n_classes-1):
      idxs = np.where(pred == idx_class)  # В массиве pred находим все элементы со значением, равным искомому классу idx_class
      idxs = idxs[0]                      # Размерность полученного маасива в np.where иммет тип (x,). Оставляем только первую размерность
      if (len(idxs) == 0):                # Если элементы искомого класса не найдены,
        continue                          # то переходим к поиску команд следующего класса

      curr = [] # Временный массив для хранения информации о найденных командах
      '''
      в массиве idx данные прдеставлены следующим образом:
      [4, 5, 6, 7, 123, 124, 125, 126, 127]
      в массив curr мы запишем [4, 123] только стартовые индексы
      поскольку очевидно, что 4,5,6,7 и 123,124,125,126,127 представляют единую команду
      '''
      curr_idx =int(idxs[0])  # Текущий стартовый индекс
      summ, length = 0, 0     # summ - хранит сумму вероятностей, с которой сеть отнесла команду к данному классу; length - длина последовательно идущих элементов для одной команды (для массива curr из примера                                                                        
                              #[4, 123] длина соответствующая первому элементу будет 4, второму - 5 )
      
      for i in range(len(idxs)):              # Пробегаем по всему массиву idxs
        summ += g_pred[idxs[i]][idx_class]    # Считаем сумму вероятности
        length += 1                           # Увеличиваем длинну последовательности
        if i == len(idxs)-1:                  # Если последний элемент последовательности
          if (length >= min_count and summ / length >= rate):   # Проверяем на условия разбора: длина последовательности должна быть больше входного параметра min_count
                                                                # summ / length должно быть больше входного параметра rate
            curr.append([curr_idx, length, summ / length])      # Если условия выполняются, то добавляем в маасив стартовый индекс найденной команды, длинну последовательности и summ / length

          break  
        if idxs[i+1]-idxs[i]>hole:                            # Если следующий индекс больше текущего на hole (означает, что следующий элемент относится уже к другой комманде)
          if (length >= min_count and summ / length >= rate): # Проверяем на условия разбора: длина последовательности должна быть больше входного параметра min_count
                                                              # summ / length должно быть больше входного параметра rate
 #           print(length)
            curr.append([curr_idx, length, summ / length])  # Если условия выполняются, то добавляем в маасив стартовый индекс найденной команды, длинну последовательности и summ / length
          curr_idx = int (idxs[i+1])                        # Изменяем текущий стартовый индекс
          summ, length = 0, 0                               # Обнуляем summ и length
      curr_audio = [] 
      for elem in curr:                                     # Проходим по всему массиву curr
        curr_audio = audio_full[elem[0]]      # Если это стартовый элемент исходных данных, то берем самую первую mfcc
        for j in range(1,elem[1]):            # Пробегаем цикл от 1 до elem[1]+1 (где elem[1] хранит длинну последовательности элементов, отнесенных к одной команде)
         if (elem[0]+j == len(audio_full)):   # Если elem[0] + j равно длинне mfcc, то выходим из цикла
           break
         curr_audio = np.hstack((curr_audio, audio_full[elem[0]+j][-step_mfcc:]))
        curr_audio = np.array(curr_audio)             # Переводим массив в numpy
        out.append([curr_audio, idx_class, elem])  # Добавляем данные в выходной массив
    return out, pred                         # Возращаем массив с данными, массив с классами команд, массив с softmax данными

  проверочная_выборка = '/content/умный_дом/test_speech/'
  список_проверочной = os.listdir(проверочная_выборка)
  wavfiles =  проверочная_выборка+список_проверочной[np.random.randint(len(список_проверочной))]  # Получаем имя случайного файла
  print ('Исходное аудио для проверки работы нейронной сети: ')
  y_org,sr = librosa.load(wavfiles)      
  display.display(display.Audio (y_org, rate = 22050))
  print()

  out, pred = predict(wavfiles, model=модель, min_count = 8, rate = порог, hole = 1)                # Вызываем predict для очередного файла  
  color = ['y','g','b']
  if (len(out)==0):                                                                                   # Если длина массива равна 0, то команда не распознана
    print('Команда не распознанана!!!')
    return
  for elem in out[:1]: 
    print ('Распознана команда: "', classes[elem[1]], '" (вероятность - %.2f' % (elem[2][2]*100), '%)')  # Выводим название
    display.display(display.Audio(elem[0], rate = 22050))
    fig = plt.figure(figsize=(30,5))
    ax = fig.add_subplot(111) 
    plt.plot(y_org)
    ax.add_patch(Rectangle((441*elem[2][0], 1.), 
                            20000, -2, 
                              fc =color[elem[1]],  
                              ec =color[elem[1]], 
                              fill=True,
                              lw = 2,
                              alpha=0.5) ) 

def параметризация_аудио(файл):
  файл = файл.lower()
  d ={'кондиционер': '1_cond',
      'свет':'2_light',
      'телевизор':'3_tv',
      'фон':'4_noise'}
  assert файл in d, f'Команда распознана неверно. Возможно вы имели ввиду \'{check_for_errors.check_word(файл, "home")}\'.'

  выборка = '/content/умный_дом/comands/'+d[файл]+'/'
  список = os.listdir(выборка)
  wavfiles =  выборка+список[np.random.randint(len(список))]  # Получаем имя случайного файла
  print('Команда:', файл)
  print ('Исходное аудио: ')
  y_org,sr = librosa.load(wavfiles)  
  plt.figure(figsize=(30, 5))
  ipd.display(display.Audio(data=y_org, rate = sr)) 
  librosa.display.waveplot(y_org)
  plt.show()
  print()
  print()  
  
  print('Спектр сигнала:')
  X = librosa.stft(y_org)
  Xdb = librosa.amplitude_to_db(abs(X))      
  plt.figure(figsize=(10, 6))
  librosa.display.specshow(Xdb, sr=sr, x_axis='time', y_axis='hz')
  plt.colorbar()
  plt.show() 
  print()
  print
  #Вычисляем и отображаем Мел-частотные кепстральные коэффициенты  
  mfccs = librosa.feature.mfcc(y_org, sr=sr)
    #Нормируем Мел коэффициенты
  mfccs = sklearn.preprocessing.scale(mfccs, axis=1)
  #Отображаем нормированные коэффициенты
  hop_length = 512 #Задаём размер отрезка сигнала, по которому расcчитывается частоты цветности
  #Расчитываем и отображаем частоту цветности
  chromagram = librosa.feature.chroma_stft(y_org, sr=sr, hop_length=hop_length)




print('Идет проверка и установка дополнительных библиотек, необходимых для работы terra_ai')
print('Это может занять несколько минут...')
import subprocess, os, warnings, time
from pandas.core.common import SettingWithCopyWarning
from subprocess import STDOUT, check_call
from IPython import display
import numpy as np
import requests
import librosa #Для параметризации аудио
import ast
import json
from tabulate import tabulate
import getpass

def выполнить_команду(команда='!ls'):
  proc = subprocess.Popen(f'{команда}', shell=True, stdin=None, stdout=open(os.devnull,"wb"), stderr=STDOUT, executable="/bin/bash")
  proc.wait()
  pass
import os
import time
import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3' 
from tensorflow.keras.utils import to_categorical
import tensorflow as tf
tf.get_logger().setLevel("WARNING")
import logging
logging.getLogger("tensorflow").setLevel(logging.WARNING)
#tf.get_logger().setLevel(logging.ERROR)
from tensorflow.keras.utils import to_categorical
выполнить_команду('pip -q install pymorphy2') 
выполнить_команду('pip -q install scikit-learn') 
выполнить_команду('pip install python-Levenshtein') 
import Levenshtein
import sklearn
from sklearn.model_selection import train_test_split #Разбиение на обучающую и проверочную выборку
from sklearn.preprocessing import LabelEncoder, StandardScaler #Для нормировки данных

from . import датасет, констр, модель, сегментация, traide, ряды, прачечная, обработка_текста, сегментация_договоров, квартиры, home, check_for_errors
from . import модель_для_проверки_дз
import seaborn as sns
sns.set_style('darkgrid')
warnings.simplefilter(action="ignore", category=SettingWithCopyWarning)

display.clear_output(wait=True)
print('Все необходимые библиотеки установлены и готовы к работе')
  
def загрузить_базу(база = ''):  
  return датасет.загрузить_базу(
    база = база)

#Генетика 

def загрузить_модули():
  print('Выполняется загрузка и установка модулей')
  print('Это может занять несколько минут...')
  выполнить_команду('apt install swig cmake libopenmpi-dev zlib1g-dev > /dev/null 2>&1')
  выполнить_команду('pip -q install stable-baselines==2.5.1 box2d box2d-kengz > /dev/nul 2>&1')
  выполнить_команду('pip -q install gym pyvirtualdisplay > /dev/null 2>&1')
  выполнить_команду('pip -q install xvfbwrapper > /dev/null 2>&1')
  выполнить_команду('apt-get update > /dev/null 2>&1')
  выполнить_команду('sudo apt-get install xvfb > /dev/null 2>&1')
  выполнить_команду('apt-get install xdpyinfo > /dev/null 2>&1')
  display.clear_output(wait=True)
  from . import генетика
  print('Все модули установлены и готовы к работе')

def создать_дисплей(width, hight):
  from . import генетика
  ширина = width
  высота = hight
  генетика.создать_дисплей(ширина, высота)

def показать_пример(task):
  from . import генетика
  задача = task
  генетика.показать_пример(задача)

def обучить_алгоритм(размер_популяции, количество_эпох, коэфицент_мутации, количество_выживших):
  from . import генетика
  генетика.обучить_алгоритм(размер_популяции, количество_эпох, коэфицент_мутации, количество_выживших)

def посадить_корабль():
  from . import генетика
  генетика.посадить_корабль()

##########################

#Демонстрация
def конструктор():
  return констр.конструктор()

def демонстрация_МНИСТ():
  return модель.демонстрация_МНИСТ()

def показать_изменение_маски_MNIST(number):
  return модель.показать_изменение_маски_MNIST(number)

def показать_изменение_веса():
  return модель.показать_изменение_веса()

def демонстрация_АВТО():
  return модель.демонстрация_АВТО()

def показать_изменение_маски_АВТО(num):
  return модель.показать_изменение_маски_АВТО(num)

def показать_маски():
  import logging
  tf.get_logger().setLevel(logging.ERROR)
  return модель.показать_маски()
#LabStory
# Авторизация
def авторизация_LabStory():
  return модель.авторизация_LabStory()  
  
# Добавление набора данных
def добавить_датасет_LabStory(dataset_dict):
  return модель.добавить_датасет_LabStory(dataset_dict)
  
# Получение списка набора данных
def список_датасетов_LabStory():
  return модель.список_датасетов_LabStory()

# Удаление набора данных
def удалить_датасет_LabStory(id):
  return модель.удалить_датасет_LabStory(id)

#Выбор датасета по id
def выбрать_датасет_LabStory(id):
  return модель.выбрать_датасет_LabStory(id)

# Вывод информации о текущем датасете
def текущий_датасет():
  return модель.текущий_датасет()

# Создание задачи    
def добавить_задачу_LabStory(task_dict):
  return модель.добавить_задачу_LabStory(task_dict)

# Получение списка задач 
def список_задач_LabStory():
  return модель.список_задач_LabStory()

# Выбор задачи по id    
def выбрать_задачу_LabStory(id):
  return модель.выбрать_задачу_LabStory(id)

def текущая_задача():
  return модель.текущая_задача()

# Сохранение эксперимента
def сохранить_эксперимент_LabStory(experiment_dict):
  return модель.сохранить_эксперимент_LabStory(experiment_dict)
  
def сохранить_эксперимент_LabStory_terra_ai(experiment_dict):
  return модель.сохранить_эксперимент_LabStory_terra_ai(experiment_dict)

# Получение списка экспериметов
def список_экспериметов_LabStory():
  return модель.список_экспериметов_LabStory()

# Получение эксперимета по id  
def посмотреть_эксперимент_по_id_LabStory(id):
  return модель.посмотреть_эксперимент_по_id_LabStory(id)
  
def все_эксперименты_по_задаче(id):
    return модель.все_эксперименты_по_задаче(id)

def получить_архитектуру(id_exp):
    return модель.получить_архитектуру(id_exp)
    
## Прачечная
def ввод_данных():
  прачечная.ввод_данных()
def рассчитать_план_на_день(общее_число_ботов, количество_выживших, количество_эпох, коэффициент_мутаций):
  прачечная.рассчитать_план_на_день(общее_число_ботов, количество_выживших, количество_эпох, коэффициент_мутаций)
def план_на_день():
  прачечная.план_на_день()
def информация_о_мешке(n=None):
  прачечная.информация_о_мешке(n)
  
### ДАТАСЕТ
def показать_примеры(**kwargs):
  if 'база' in kwargs:
    kwargs['путь'] = kwargs['база']
  датасет.показать_примеры(**kwargs)

def показать_пример_договора(количество = 1):
    сегментация_договоров.показать_пример(количество)

def показать_пример_временного_ряда(база, старт, финиш):
    return ряды.показать_примеры(база, старт, финиш)

def показать_пример_квартиры(количество = 1):
    квартиры.samples(количество)

def показать_примеры_голосовых_команд():
    home.показать_примеры_голосовых_команд()

def показать_примеры_для_трейдинга(данные):
    traide.show_full_data(данные)

def показать_примеры_для_интервала(данные, start, end):
    traide.show_data(данные, start, end)

def предобработка_данных(**kwargs):
  return датасет.предобработка_данных(**kwargs)

def получить_входной_размер(arr):
  return arr.shape[1:]

def получить_выходной_размер(arr):
  return arr.max()+1

def создать_выборки(**kwargs):
  if 'база' in kwargs:
      kwargs['путь'] = kwargs['база']
  assert 'путь' in kwargs, f'Используйте параметр \'путь\', чтобы уточнить базу из которой вы хотите создать выборку.'
  data_list_seg = {'самолеты':(176, 320), 'самолеты_макс':(520, 960), 'губы':(192,256), 'люди': (256, 384)}
  data_list_gen = ['квартиры', 'трафик', 'вакансии', 'диалоги', 'повышение_размерности', 'договоры']
  kwargs['путь'] = kwargs['путь'].lower().replace('/','')
  if kwargs['путь']=='mnist':
      kwargs_time =  {'путь':kwargs['путь']}
      return датасет.создать_выборки(**kwargs)

  датасет.func_true_false(kwargs['путь'].replace('/',''), 'dataset')
  
  if kwargs['путь'] in ['tesla','симптомы_заболеваний']:
      kwargs_time =  {'путь':kwargs['путь'], 'MWC':10000,'xLen':50,'step':50}
      kwargs = check_for_errors.func_dict(kwargs, kwargs_time)
      
  elif kwargs['путь'] in data_list_gen:
      kwargs =  {'путь':kwargs['путь']}
      return датасет.создать_выборки(**kwargs)
  
  elif kwargs['путь'] == 'трейдинг':
      kwargs_time =  {'путь':kwargs['путь'],'акции':'полиметаллы', 'количество_анализируемых_дней':10,'период_предсказания':1}
      kwargs = check_for_errors.func_dict(kwargs, kwargs_time)
  
  elif kwargs['путь'] == 'умный_дом':
      kwargs_time =  {'путь':kwargs['путь'],'длина':0.5, 'шаг':0.04}
      kwargs = check_for_errors.func_dict(kwargs, kwargs_time)
      
  elif kwargs['путь'] in data_list_seg:
      some_path = kwargs['путь']
      размер = data_list_seg[f'{some_path}']
      kwargs_time =  {'путь':some_path, 'размер':размер}
      kwargs = check_for_errors.func_dict(kwargs, kwargs_time)
  
  else:
      kwargs_time =  {'путь':kwargs['путь'],'размер':(108,192), 'коэф_разделения':0.1}
      kwargs = check_for_errors.func_dict(kwargs, kwargs_time)
  return датасет.создать_выборки(**kwargs)

def создать_текстовые_выборки(путь_к_базе,MWC=10000, xLen=50, step=50):
  MWC = int(MWC)
  xLen = int(xLen)
  step = int(step)
  return обработка_текста.создать_выборки(путь_к_базе,MWC, xLen, step)

def создать_выборки_трекер(путь, размер):
  dir = путь
  size = размер
  return датасет.img_loader(dir='/трекер', size=(64,64))

def создать_выборки_квартир():
  return квартиры.создать_выборки()

def создать_выборки_вакансии():
  return модель.создать_выборки_вакансии()

def создать_выборки_для_торговли(акции,количество_анализируемых_дней,период_предсказания):
  return traide.получить_данные(акции,
  количество_анализируемых_дней,
  период_предсказания)

def создать_выборки_голосовых_команд(длина, шаг):
  return home.создать_выборки(длина, шаг)

def создать_выборки_временного_ряда(база, период):
  return ряды.создать_выборки_трафик(база, период)

def создать_выборки_сегментация_текста(договоры):
  return сегментация_договоров.создать_выборки_договоров(договоры)
  
def создать_выборки_для_чатбота(вопросы, ответы,количество_пар=10000):
  return обработка_текста.создать_выборки_чатбота(вопросы, ответы,количество_пар)
  
def создать_выборки_для_сегментации(images_airplane, segments_airplane):
   return сегментация.create_xy(images_airplane, segments_airplane)

def создать_выборки_для_повышения_размерности():  
  return повышение_размерности.генератор_данных_DIV2K()

# МОДЕЛЬ
def схема_модели(мод):
  return модель.схема_модели(мод)
  

def обучение_модели_квартиры(мод, x_train, y_train, x_test=[], y_test=[], размер_пакета=None, количество_эпох=None, коэф_разделения = 0.2):  
  return модель.обучение_модели_квартиры(мод, x_train, y_train, x_test, y_test, размер_пакета, количество_эпох, коэф_разделения)  

def обучение_модели(мод, x_train, y_train, x_test=[], y_test=[], размер_пакета=None, количество_эпох=None, коэф_разделения = 0.2, **kwargs):
  global history
  return модель.обучение_модели(мод, x_train, y_train, x_test, y_test, размер_пакета, количество_эпох, коэф_разделения, **kwargs)

def обучение_модели_временного_ряда(мод, ген1, ген2, количество_эпох=None):
  модель.обучение_модели_трафик(мод, ген1, ген2, количество_эпох) 
  return history

def предобучение_генератора(генератор, обучающая_выборка, проверочная_выборка, количество_шагов, интервал_вывода):
  повышение_размерности.предобучение_генератора(генератор,обучающая_выборка, проверочная_выборка, количество_шагов, интервал_вывода)
  print('Предобучение генератора завершено')
  
def обучение_модели_повышения_размерности(нейронка,обучающая_выборка):
  нейронка.train(обучающая_выборка, steps=5000) #Обучаем Srgan полностью

def обучение_модели_обнаружения(количество_эпох):
  обнаружение.обучение_модели_обнаружения(количество_эпох)

def тест_модели_классификации(нейронка, тестовый_набор, правильные_ответы, классы, количество=1):
  модель.тест_модели_классификации(нейронка, тестовый_набор, правильные_ответы, классы, количество)

def тест_модели_трекер(нейронка, тестовый_набор, правильные_ответы, классы, количество=5):
  модель.тест_модели_трекер(нейронка, тестовый_набор, правильные_ответы, классы, количество)

def тест_модели_вакансии(нейронка, x_val, y_val):
  return модель.тест_модели_вакансии(нейронка, x_val, y_val)

def тест_модели_симптомы(нейронка, xLen, step, симптомы, классы):
  xLen = int(xLen)
  step = int(step)
  return обработка_текста.тест_модели_симп(нейронка, xLen, step, симптомы, классы)

def тест_модели_отзывы(нейронка, xLen, step, отзыв, классы):
    симптомы = отзыв
    xLen = int(xLen)
    step = int(step)
    обработка_текста.тест_модели_симп(нейронка, xLen, step, симптомы, классы)

def тест_модели_писатели(нейронка, xLen, step, текст, классы):
    обработка_текста.тест_модели_писатели(нейронка, xLen, step, текст, классы)
    
def тест_модели_на_своем_изображении(нейронка, размер_изображения, классы):
  модель.тест_на_своем_изображении(нейронка, размер_изображения, классы)

def тест_модели_сегментации(мод, тестовые_изображения,  **kwargs):
  сегментация.тест_модели(мод, тестовые_изображения,  **kwargs)

def тест_модели_сегментации_договоров(мод, теги, договора):
  сегментация_договоров.тест_модели(мод, теги, договора)

def тест_модели_торговли(нейронка,тестовая_выборка, метки_тестовой_выборки, период_предсказания,количество_анализируемых_дней):
  traide.model_test(
    нейронка, 
    тестовая_выборка, метки_тестовой_выборки,
    период_предсказания,
    количество_анализируемых_дней)

def примеры_распознавания(нейронка_1, период_предсказания,количество_анализируемых_дней):
    traide.example_traid(нейронка_1, период_предсказания,количество_анализируемых_дней)

def тест_модели_повышения_размерности(нейронка):
  модель.тест_модели_HR(нейронка)

def торговля(нейронка_1, тестовая_выборка, тип):
  traide.traiding(нейронка_1, тестовая_выборка, тип)

def тест_чат_бота(нейронка, размер_словаря, энкодер, декодер):
  обработка_текста.тест_модели_чат_бот(нейронка, размер_словаря, энкодер, декодер)
  
def тест_обученного_чат_бота():
  обработка_текста.загрузить_предобученную_модель()
  
def тест_модели_голосовых_команд(нейронка, порог, длина, шаг):
  home.тест_модели(нейронка, порог, длина, шаг)

def тест_модели_обнаружения_людей(нейронка):
  обнаружение.тест_модели()

def тест_модели_временного_ряда(мод, данные):
  ряды.тест_модели_трафика(мод, данные)

def тест_модели_квартиры(нейронка, *данные,):
  квартиры.тест_модели(нейронка, *данные,)

def создать_сеть(слои, входной_размер, задача, параметры_модели=None):
  return модель.создать_сеть(слои, входной_размер, параметры_модели, задача)

def создать_составную_сеть(данные, метки, *нейронки):
  return модель.создать_составную_сеть(данные, метки, *нейронки)

def создать_составную_сеть_квартиры(данные, *нейронки):
  return модель.создать_составную_сеть_квартиры(данные, *нейронки)

def создать_модель_Sequence_to_Sequence(размер_словаря, энкодер, декодер):
  return модель.создать_сеть_чат_бот(размер_словаря, энкодер, декодер)

def создать_генератор_повышения_размерности(стартовый_блок, основной_блок, финальный_блок):
  мод = модель.создать_генератор_повышения_размерности(стартовый_блок, основной_блок, финальный_блок)
  предобучение_генератора = повышение_размерности.SrganGeneratorTrainer(мод, checkpoint_dir=f'folder')    
  print('Модель генератора создана')    
  return предобучение_генератора

def создать_дискриминатор_повышения_размерности(блок_дискриминатора,количество_блоков, финальный_блок):
  return модель.создать_дискриминатор_повышения_размерности(блок_дискриминатора,количество_блоков, финальный_блок)

def создать_PSP(**kwargs):
  return модель.создать_PSP(**kwargs)

def создать_UNET(**kwargs):
  return модель.создать_UNET(**kwargs)

def создать_модель_повышения_размерности(генератор, дискриминатор):
  return повышение_размерности.создать_модель(генератор, дискриминатор)

def загрузить_модель_повышения_размерности():
  return повышение_размерности.загрузить_веса_готовой_модели()

def загрузка_обученной_модели_обнаружения_людей():
  return обнаружение.cоздать_модель_YOLO()  

def параметризация_аудио(файл):
  home.параметризация_аудио(файл)

###
##Проверка ДЗ
###

def дз_первый_день(**kwargs):
    check_for_errors.test_hw_1(**kwargs)

def дз_второй_день(**kwargs):
    check_for_errors.test_hw_2(**kwargs)

def дз_третий_день(**kwargs):
    check_for_errors.test_hw_3(**kwargs)





import os, random, io, json

from tensorflow.keras.preprocessing.text import Tokenizer # Методы для работы с текстами и преобразования их в последовательности
from tensorflow.keras.preprocessing.text import tokenizer_from_json
import numpy as np
from gensim.models import word2vec # Импортируем gensim 

def показать_пример(количество):
    lst = os.listdir('Договоры/Договора432/')
    rnd = random.choice(lst)
    f = open('Договоры/Договора432/'+rnd, 'r', encoding='utf-8')
    for line in f:
        print(line)

def создать_выборки_договоров(
      договора,      
      xLen = 256, # Длина окна  по умолчанию
      step = 30, # Шаг  по умолчанию
      embeddingSize = 300): # Количество измерений для векторного пространства  по умолчанию
  agreements = договора[0]
  words = договора[1]
  wordsToTest = договора[2]
  # lower=True - приводим слова к нижнему регистру
  # char_level=False - просим токенайзер учитывать слова, а не отдельные символы
  global tags_index, clean_voc, xLen_test, step_test, embeddingSize_test, tokenizer

  xLen_test = xLen
  step_test = step
  embeddingSize_test = embeddingSize

  tokenizer = Tokenizer(lower=True, filters = '', char_level=False)

  tokenizer.fit_on_texts(words) # "Скармливаем" наши тексты, т.е. даём в обработку методу, который соберет словарь частотности
  clean_voc = {} 

  for item in tokenizer.word_index.items(): #Преобразуем полученный список 
    clean_voc[item[0]] = item[1] # В словарь, меняя местами элементы полученного кортежа 
  # Преобразовываем текст в последовательность индексов согласно частотному словарю
  tok_agreem = tokenizer.texts_to_sequences(words) # Обучающие тесты в индексы

  # Собираем список индексов и dummy encoded вектора
  def get01XSamples(tok_agreem, tags_index):
    tags01 = [] # Список для тегов
    indexes = [] # Здесь будут лежать индексы
  
    for agreement in tok_agreem: # Проходимся по каждому договору-списку
      tag_place = [0, 0, 0, 0, 0, 0] # Создаем вектор [0,0,0,0,0,0]
      for ex in agreement: # Проходимся по каждому слову договора
          if ex in tags_index: # Смотрим, если индекс оказался нашим тегом
            place = np.argwhere(tags_index==ex) # Записываем под каким местом лежит этот тег в своем списке
            if len(place)!=0: # Проверяем, чтобы тег действительно был
              if place[0][0]<6: # Первые шесть тегов в списке - открывающие
                tag_place[place[0][0]] = 1    # Поэтому ставим 1
              else: 
                tag_place[place[0][0] - 6] = 0  # Остальные теги закрывающие, так что меняем на ноль
          else:          
            tags01.append(tag_place.copy()) # Расширяем наш список с каждой итерацией. Получаем в конце длинный список из всех тегов в одном 
            indexes.append(ex) # Докидываем индекс слова в список индексов

    return indexes, tags01

  # Получение списка слов из индексов
  def reverseIndex(clean_voc, x):
    reverse_word_map = dict(map(reversed, clean_voc.items())) # Берем пары значений всего словаря и размечаем наоборот, т.е. value:key
    words = [reverse_word_map.get(letter) for letter in x] # Вытаскиваем по каждому ключу в список
    return words # Возвращаем полученный текст

  # Формируем выборку из индексов
  def getSetFromIndexes(wordIndexes, xLen, step): 
    xBatch = [] # Лист для фрагментов текста
    wordsLen = len(wordIndexes) # Определяем длинну текста
    index = 0 # Задаем стартовый индекс
    
    while (index + xLen <= wordsLen): # Пока сумма индекса с длинной фрагмента меньше или равна числу слов в выборке
      xBatch.append(wordIndexes[index:index+xLen]) # Добавляем X в лист фразментов текста
      index += step # Сдвигаемся на step

    return xBatch # Лист для фрагментов текста   

  # s1 Условия
  # s2 Запреты
  # s3 Стоимость, всё про цены и деньги
  # s4 Всё про сроки
  # s5 Неустойка
  # s6 Всё про адреса и геолокации
  tags_index = ['<s' + str(i) + '>' for i in range(1, 7)] # Получаем список открывающих тегов
  closetags = ['</s' + str(i) + '>' for i in range(1, 7)] # Получаем список закрывающих тегов
  tags_index.extend(closetags) # Объединяем все теги

  tags_index = np.array([clean_voc[i] for i in tags_index]) # Получаем из словаря частотности индексы всех тегов

  xData, yData = get01XSamples(tok_agreem,tags_index) # Распознаем теги и создаем список с ними, с индексами
  decoded_text = reverseIndex(clean_voc, xData) # Для создания списков с embedding-ами сначала преобразуем список индексов обратно в слова
    
 
    # Генерируем наборы с заданными параметрами окна
  xTrain = getSetFromIndexes(decoded_text, xLen, step) # Последовательность из xLen слов
  yTrain = getSetFromIndexes(yData, xLen, step) # Последовательность из xLen-тегов

    # Создаем выборки
  def getSets(model, senI, tagI):
    xVector = [] # Здесь будет лежать embedding представление каждого из индексов
    tmp = [] # Временный список
    for text in senI: # Проходимся по каждому тексту-списку
      tmp=[]
      for word in text: # Проходимся по каждому слову в тексте-списке
        tmp.append(model[word]) 

      xVector.append(tmp)

    return np.array(xVector), np.array(tagI)

      # Передаем в word2vec списки списков слов для обучения
  # size = embeddingSize - размер эмбеддинга
  # window = 10 - минимальное расстояние между словами в эмбеддинге 
  # min_count = 1 - игнорирование всех слов с частотой, меньше, чем 1
  # workers = 10 - число потоков обучения эмбеддинга
  # iter = 10 - число эпох обучения эмбеддинга

  modelGENSIM = word2vec.Word2Vec(xTrain, size = embeddingSize, window = 10, min_count = 1, workers = 10, iter = 10)

  xTrainGENSIM, yTrainGENSIM = getSets(modelGENSIM, xTrain, yTrain)
  tokenizer_json1 = tokenizer.to_json()
  with io.open('tokenizer_dog.json', 'w', encoding='utf-8') as f:
    f.write(json.dumps(tokenizer_json1, ensure_ascii=False))
  
  with open('clean_voc.json', 'w') as fp:
    json.dump(clean_voc, fp)
    
  return xTrainGENSIM, yTrainGENSIM, tags_index   #, xLen, embeddingSize
  
def тест_модели(model, теги, договора):
  # получаем нужные параметры из функции формировавшие xtrain, yTrain
  xLen = 256
  step = 30
  embeddingSize = 300
  tags_index = теги
  with open('tokenizer_dog.json') as f:
    data = json.load(f)
    tokenizer = tokenizer_from_json(data)
    
  with open('clean_voc.json') as json_file: 
    clean_voc = json.load(json_file) 
    
  print("Тест модели:")
  
  wordsToTest = договора[2]
 # Функция, выводящая точность распознавания каждой категории отдельно 
  def recognizeSet(tagI, pred, tags, length, value):
    total=0

    for j in range(6): # общее количество тегов
      correct=0
      for i in range(len(tagI)): # проходимся по каждому списку списка тегов
        for k in range(length): # проходимся по каждому тегу
          if tagI[i][k][j]==(pred[i][k][j]>value).astype(int): # если соответствующие индексы совпадают, значит сеть распознала верно
            correct+=1 
      print("Сеть распознала категорию '{}' на {}%".format(tags[j],round(100*correct/(len(tagI)*length), 2)))
      total += 100 * correct / (len(tagI)*length)
    print("Cредняя точность {}%".format(round(total/6,2)))

  def get01XSamples(tok_agreem, tags_index):
    tags01 = [] # Список для тегов
    indexes = [] # Здесь будут лежать индексы
  
    for agreement in tok_agreem: # Проходимся по каждому договору-списку
      tag_place = [0, 0, 0, 0, 0, 0] # Создаем вектор [0,0,0,0,0,0]
      for ex in agreement: # Проходимся по каждому слову договора
          if ex in tags_index: # Смотрим, если индекс оказался нашим тегом
            place = np.argwhere(tags_index==ex) # Записываем под каким местом лежит этот тег в своем списке
            if len(place)!=0: # Проверяем, чтобы тег действительно был
              if place[0][0]<6: # Первые шесть тегов в списке - открывающие
                tag_place[place[0][0]] = 1    # Поэтому ставим 1
              else: 
                tag_place[place[0][0] - 6] = 0  # Остальные теги закрывающие, так что меняем на ноль
          else:          
            tags01.append(tag_place.copy()) # Расширяем наш список с каждой итерацией. Получаем в конце длинный список из всех тегов в одном 
            indexes.append(ex) # Докидываем индекс слова в список индексов

    return indexes, tags01

  # Формируем выборку из индексов
  def getSetFromIndexes(wordIndexes, xLen, step): 
    xBatch = [] # Лист для фрагментов текста
    wordsLen = len(wordIndexes) # Определяем длинну текста
    index = 0 # Задаем стартовый индекс
    
    while (index + xLen <= wordsLen): # Пока сумма индекса с длинной фрагмента меньше или равна числу слов в выборке
      xBatch.append(wordIndexes[index:index+xLen]) # Добавляем X в лист фразментов текста
      index += step # Сдвигаемся на step

    return xBatch # Лист для фрагментов текста   

  # Получение списка слов из индексов
  def reverseIndex(clean_voc, x):
    reverse_word_map = dict(map(reversed, clean_voc.items())) # Берем пары значений всего словаря и размечаем наоборот, т.е. value:key
    words = [reverse_word_map.get(letter) for letter in x] # Вытаскиваем по каждому ключу в список
    return words # Возвращаем полученный текст

  # Создаем выборки
  def getSets(model, senI, tagI):
    xVector = [] # Здесь будет лежать embedding представление каждого из индексов
    tmp = [] # Временный список
    for text in senI: # Проходимся по каждому тексту-списку
      tmp=[]
      for word in text: # Проходимся по каждому слову в тексте-списке
        tmp.append(model[word]) 

      xVector.append(tmp)

    return np.array(xVector), np.array(tagI)

  # Преобразовываем текст в последовательность индексов согласно частотному словарю
  #tokenizer = Tokenizer(lower=True, filters = '', char_level=False)
  tok_agreemTest = tokenizer.texts_to_sequences(wordsToTest) # Обучающие тесты в индексы

  xDataTest, yDataTest = get01XSamples(tok_agreemTest,tags_index) # Распознаем теги и создаем список с ними, с индексами
  decoded_text = reverseIndex(clean_voc, xDataTest) # Для создания списков с embedding-ами сначала преобразуем список индексов обратно в слова  

  # Генерируем наборы с заданными параметрами окна
  xTest = getSetFromIndexes(decoded_text, xLen, step) # Последовательность из xLen слов
  yTest = getSetFromIndexes(yDataTest, xLen, step) # Последовательность из xLen-тегов

  # Передаем в word2vec списки списков слов для обучения
  # size = embeddingSize - размер эмбеддинга
  # window = 10 - расстояние между текущим и прогнозируемым словом в предложении
  # min_count = 1 - игнорирование всех слов с частотой, меньше, чем 1
  # workers = 10 - число потоков обучения эмбеддинга
  # iter = 10 - число эпох обучения эмбеддинга

  modelGENSIM = word2vec.Word2Vec(xTest, size = embeddingSize, window = 10, min_count = 1, workers = 10, iter = 10)  

  xTestGENSIM, yTestGENSIM = getSets(modelGENSIM, xTest, yTest)

  tags = ['S1', 'S2', 'S3', 'S4', 'S5', 'S6']
  # s1 Условия
  # s2 Запреты
  # s3 Стоимость, всё про цены и деньги
  # s4 Всё про сроки
  # s5 Неустойка
  # s6 Всё про адреса и геолокации

  pred = model.predict(xTestGENSIM) # сделаем предсказание

  recognizeSet(yTestGENSIM, pred, tags, xLen, 0.9)

  pass
####################################################################################